#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomHinduHolidays, _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.groups import (
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
    SUN_TO_NEXT_MON,
)


class Guyana(
    ObservedHolidayBase,
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
):
    """Guyana holidays.

    References:
        * [Public Holidays Amendment Act 1967](https://web.archive.org/web/20241125073357/https://parliament.gov.gy/documents/bills/11264-bill_6_of_1967_public_holidays.pdf)
        * [Public Holidays Amendment Act 1969](https://web.archive.org/web/20250424211718/https://parliament.gov.gy/documents/bills/11891-bill_26_of_1969_1.pdf)
        * [Public Holidays Act Consolidated as of 2012](https://web.archive.org/web/20250608202320/https://mola.gov.gy/laws/Volume%206%20Cap.%2018.01%20-%2023.011696968337.pdf)
        * [2021-2025](https://web.archive.org/web/20250614200338/https://moha.gov.gy/public-holidays/)

    The holidays below are not part of the latest Public Holidays Act but have been notified
    every year as a holiday by Gazette notifications for many years. The earliest and latest
    available notifications for each are linked below.

    * Independence Day:
        * [Extraordinary Gazette 14 May 2016](https://web.archive.org/web/20250210091920/https://officialgazette.gov.gy/images/gazettes-files/Extraordinary-gazette_14may161.pdf)
        * [Extraordinary Gazette 10 May 2025](https://web.archive.org/web/20250611200312/https://officialgazette.gov.gy/images/gazette2025/may/Extra_10MAY2025NotiPHA.pdf)
    * CARICOM Day:
        * [Extraordinary Gazette 29 June 2016](https://web.archive.org/web/20250209205727/https://officialgazette.gov.gy/images/gazettes-files/Extraordinary-gazette_29jun16.pdf)
        * [Extraordinary Gazette 22 June 2024](https://web.archive.org/web/20250209154128/https://officialgazette.gov.gy/images/gazette2024/jun/Extra_22JUNE2024NotiPHA.pdf)
    * Arrival Day:
        * [Extraordinary Gazette 16 April 2019](https://web.archive.org/web/20221118182239/https://officialgazette.gov.gy/images/gazette2019/apr/Extra_20APRIL2019NotPubHol.pdf)
        * [Extraordinary Gazette 26 April 2025](https://web.archive.org/web/20250611200310/https://officialgazette.gov.gy/images/gazette2025/apr/Extra_26APRIL2025PHA.pdf)
    * Commonwealth Day / Emancipation Day:
        * No amendment act removing Commonwealth Day and adding Emancipation Day it has been that
          way in annual Gazette notifications for years.
        * [Extraordinary Gazette 21 July 2016](https://web.archive.org/web/20250211095036/https://officialgazette.gov.gy/images/gazettes-files/Extraordinary-gazette_21jul16.pdf)
        * [Extraordinary Gazette 16 July 2022](https://web.archive.org/web/20231214011437/https://officialgazette.gov.gy/images/gazette2022/jul/Extra_16JULY2022NotificPHA.pdf)
    """

    country = "GY"
    default_language = "en_GY"
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    # %s (observed).
    observed_label = tr("%s (observed)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (observed, estimated)")
    start_year = 1968
    supported_languages = ("en_GY", "en_US")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.

        In Guyana, the dates of the Islamic calendar usually fall a day later than
        the corresponding dates in the Umm al-Qura calendar.
        """
        ChristianHolidays.__init__(self)
        HinduCalendarHolidays.__init__(self, cls=GuyanaHinduHolidays)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self,
            cls=GuyanaIslamicHolidays,
            show_estimated=islamic_show_estimated,
            calendar_delta_days=+1,
        )
        StaticHolidays.__init__(self, cls=GuyanaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")), rule=SAT_SUN_TO_NEXT_MON)

        if self._year <= 1969 or self._year >= 2016:
            # Independence Day.
            self._add_holiday_may_26(tr("Independence Day"))

        if self._year >= 1970:
            # Republic Day.
            self._add_holiday_feb_23(tr("Republic Day"))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Labor Day.
        self._add_observed(self._add_labor_day(tr("Labour Day")))

        if self._year >= 2019:
            # Arrival Day.
            self._add_observed(self._add_holiday_may_5(tr("Arrival Day")))

        if self._year >= 2016:
            # CARICOM Day.
            self._add_holiday_1st_mon_of_jul(tr("CARICOM Day"))

        if self._year >= 2016:
            # Emancipation Day.
            self._add_observed(self._add_holiday_aug_1(tr("Emancipation Day")))
        else:
            # Commonwealth Day.
            self._add_holiday_1st_mon_of_aug(tr("Commonwealth Day"))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Day after Christmas.
            self._add_christmas_day_two(tr("Day after Christmas")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        # Holi.
        self._add_observed(self._add_holi(tr("Phagwah")))

        # Diwali.
        self._add_observed(self._add_diwali_india(tr("Deepavali")))

        # Prophet's Birthday.
        for dt in self._add_mawlid_day(tr("Youman Nabi")):
            self._add_observed(dt)

        # Eid al-Adha.
        for dt in self._add_eid_al_adha_day(tr("Eid-Ul-Azha")):
            self._add_observed(dt)


class GY(Guyana):
    pass


class GUY(Guyana):
    pass


class GuyanaHinduHolidays(_CustomHinduHolidays):
    # https://web.archive.org/web/20250428060218/https://www.timeanddate.com/holidays/guyana/deepavali
    DIWALI_INDIA_DATES = {
        2016: (OCT, 30),
        2017: (OCT, 19),
        2018: (NOV, 7),
        2019: (OCT, 27),
        2020: (NOV, 14),
        2021: (NOV, 4),
        2022: (OCT, 24),
        2023: (NOV, 12),
        2024: (OCT, 31),
        2025: (OCT, 20),
    }

    # https://web.archive.org/web/20250324205940/https://www.timeanddate.com/holidays/guyana/phagwah
    HOLI_DATES = {
        2005: (MAR, 26),
        2006: (MAR, 15),
        2007: (MAR, 4),
        2008: (MAR, 22),
        2009: (MAR, 11),
        2010: (MAR, 1),
        2011: (MAR, 20),
        2012: (MAR, 8),
        2013: (MAR, 27),
        2014: (MAR, 17),
        2015: (MAR, 6),
        2016: (MAR, 23),
        2017: (MAR, 12),
        2018: (MAR, 2),
        2019: (MAR, 21),
        2020: (MAR, 10),
        2021: (MAR, 28),
        2022: (MAR, 18),
        2023: (MAR, 7),
        2024: (MAR, 25),
        2025: (MAR, 14),
        2026: (MAR, 3),
    }


class GuyanaIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20250424074512/https://www.timeanddate.com/holidays/guyana/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_ADHA_DATES = {
        2005: (JAN, 21),
        2006: ((JAN, 10), (DEC, 31)),
        2007: (DEC, 20),
        2012: (OCT, 26),
        2013: (OCT, 15),
        2016: (SEP, 13),
        2019: (AUG, 11),
        2020: (JUL, 31),
        2022: (JUL, 9),
    }

    # https://web.archive.org/web/20241006104125/https://www.timeanddate.com/holidays/guyana/prophet-birthday
    MAWLID_DATES_CONFIRMED_YEARS = (2001, 2025)
    MAWLID_DATES = {
        2001: (JUN, 4),
        2002: (MAY, 24),
        2005: (APR, 21),
        2007: (MAR, 31),
        2008: (MAR, 20),
        2009: (MAR, 9),
        2010: (FEB, 26),
        2013: (JAN, 24),
        2015: ((JAN, 3), (DEC, 24)),
        2020: (OCT, 29),
    }


class GuyanaStaticHolidays(StaticHolidays):
    """Guyana special holidays.

    References:
        * [Public Holiday](https://web.archive.org/web/20250207012131/https://officialgazette.gov.gy/images/gazette2020/feb/Extra_27FEBRUARY2020NotPholA.pdf)
    """

    special_public_holidays = {
        # Public Holiday.
        2020: (MAR, 2, tr("Public Holiday")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomChineseHolidays, _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, JUN, JUL, AUG, SEP, OCT, NOV
from holidays.groups import (
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_TO_NEXT_TUE,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class ChristmasIsland(
    ObservedHolidayBase,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
):
    """Christmas Island holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Christmas_Island>
        * [2007](https://web.archive.org/web/20250612072036/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2006/01-2006_Public_Holidays_2007_CI.doc)
        * [2008](https://web.archive.org/web/20240224131231/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2007/05-2007_Public_Holidays_CI.pdf)
        * [2008 Hari Raya Puasa](https://web.archive.org/web/20240331104649/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2008/03_2008_Observance_of_Hari_Raya_Puasa_2008.pdf)
        * [2009](https://web.archive.org/web/20231211180406/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2008/02-2008_2009_public_holiday_CI_gazette.pdf)
        * [2010](https://web.archive.org/web/20250612051603/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2009/2009-Gazette_5-2009-CI-Proclamation_of_2010_Special_Public_and_Bank_Holidays.pdf)
        * [2013](https://web.archive.org/web/20240805060802/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2012/2012-Gazette_7-2012-CI-Proclamation_of_2013_Public_Holidays_for_Christmas_Island.pdf)
        * [2014](https://web.archive.org/web/20240718175750/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2013/2013-Gazette_2-2013-Christmas_Island_2014_Public_Holidays.pdf)
        * [2016 Hari Raya Puasa](https://web.archive.org/web/20240222235345/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2016/2016-Gazette_4-2016-CI-Proclamation_Special_Public_and_Bank_Holidays_2016.pdf)
        * [2017](https://web.archive.org/web/20240226014639/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_gazette/files/2016/2016-Gazette_1-2016-CI-Proclamation_Special_Public_and_Bank_Holidays_2017.pdf)
        * [2019](https://web.archive.org/web/20250517064053/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_bulletins/2018/files/A37-2018.pdf)
        * [2020](https://web.archive.org/web/20240830230128/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_bulletins/2019/files/A52-2019.pdf)
        * [2021](https://web.archive.org/web/20240713155232/https://www.infrastructure.gov.au/sites/default/files/migrated/territories/indian_ocean/iot_bulletins/2020/files/a40-ci-public-holidays-2021-proclamation.pdf)
        * [2022](https://web.archive.org/web/20240626092850/https://www.infrastructure.gov.au/sites/default/files/documents/a32-2021-2022-public-holidays-christmas-island.pdf)
        * [2023](https://web.archive.org/web/20250612044842/https://www.infrastructure.gov.au/sites/default/files/documents/A06-2022-notice-proclamation-special-public-bank-holidays-2023-ci.pdf)
        * [2023 Hari Raya Haji](https://web.archive.org/web/20240804112114/https://www.infrastructure.gov.au/sites/default/files/documents/a06-2023_community_bulletin_-_change_of_public_holiday_date_for_hari_raya_haji_2023.pdf)
        * [2024](https://web.archive.org/web/20240519034837/https://www.infrastructure.gov.au/sites/default/files/documents/a11-2023-2024-public-holidays-christmas-island.pdf)
        * [2025](https://web.archive.org/web/20250610185153/https://www.infrastructure.gov.au/sites/default/files/documents/a20-2024-administrator-community-bulletin-ci-public-holidays-2025.pdf)
    """

    country = "CX"
    default_language = "en_CX"
    # %s (observed).
    observed_label = tr("%s (observed)")
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (observed, estimated)")
    supported_languages = ("en_CX", "en_US")
    start_year = 2007

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChineseCalendarHolidays.__init__(self, cls=ChristmasIslandChineseHolidays)
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=ChristmasIslandIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, ChristmasIslandStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # Australia Day.
        self._add_observed(self._add_holiday_jan_26(tr("Australia Day")))

        # Chinese New Year.
        name = tr("Chinese New Year")
        if self._year != 2020:
            self._add_observed(self._add_chinese_new_years_day(name), rule=SAT_SUN_TO_NEXT_MON_TUE)
            self._add_observed(
                self._add_chinese_new_years_day_two(name), rule=SAT_SUN_TO_NEXT_MON_TUE
            )

        # Labor Day.
        name = tr("Labour Day")
        if self._year in {2009, 2010, 2014, 2021, 2025}:
            self._add_holiday_4th_mon_of_mar(name)
        else:
            self._add_holiday_3rd_mon_of_mar(name)

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # ANZAC Day.
        self._add_observed(self._add_anzac_day(tr("ANZAC Day")))

        # Territory Day.
        self._add_holiday_1st_mon_of_oct(tr("Territory Day"))

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE,
        )

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Christmas Day")))

        # Eid al-Fitr.
        for dt in self._add_eid_al_fitr_day(tr("Hari Raya Puasa")):
            self._add_observed(dt)

        # Eid al-Adha.
        for dt in self._add_eid_al_adha_day(tr("Hari Raya Haji")):
            if self._year not in {2014, 2025}:
                self._add_observed(dt)


class ChristmasIslandChineseHolidays(_CustomChineseHolidays):
    LUNAR_NEW_YEAR_DATES_CONFIRMED_YEARS = (2007, 2023)
    LUNAR_NEW_YEAR_DATES = {
        2007: (FEB, 19),
        2009: (JAN, 27),
        2010: (FEB, 15),
        2023: (JAN, 23),
    }


class ChristmasIslandIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = ((2007, 2010), (2019, 2025))
    EID_AL_ADHA_DATES = {
        2009: (NOV, 30),
        2013: (OCT, 15),
        2014: (OCT, 5),
        2016: (SEP, 13),
        2017: (SEP, 1),
        2024: (JUN, 17),
        2025: (JUN, 7),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = ((2007, 2010), (2019, 2025))
    EID_AL_FITR_DATES = {
        2007: (OCT, 15),
        2009: (SEP, 21),
        2013: (AUG, 8),
        2014: (JUL, 28),
        2016: (JUL, 6),
        2017: (JUN, 24),
        2019: (JUN, 5),
        2023: (APR, 22),
        2025: (MAR, 31),
    }


class CX(ChristmasIsland):
    pass


class CXR(ChristmasIsland):
    pass


class ChristmasIslandStaticHolidays:
    """Christmas Island special holidays.

    References:
        * [National Day of Mourning 2022](https://web.archive.org/web/20240712013008/https://www.infrastructure.gov.au/sites/default/files/documents/03-2022-proclamation-ci-day-of-mourning.pdf)
    """

    # Chinese New Year.
    chinese_new_year = tr("Chinese New Year")

    # Eid al-Adha.
    eid_al_adha = tr("Hari Raya Haji")

    special_public_holidays = {
        # National Day of Mourning for Queen Elizabeth II.
        2022: (SEP, 22, tr("National Day of Mourning for Queen Elizabeth II")),
    }

    special_public_holidays_observed = {
        2014: (OCT, 7, eid_al_adha),
        2020: (
            (JAN, 28, chinese_new_year),
            (JAN, 29, chinese_new_year),
        ),
        2025: (JUN, 6, eid_al_adha),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class Kiribati(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Kiribati holidays.

    References:
        * [Public Holidays Ordinance](https://web.archive.org/web/20250130150749/https://www.paclii.org/ki/legis/consol_act/pho216/)
        * [Public Holidays (Amendment) Act 2002](https://web.archive.org/web/20250130161614/https://www.paclii.org/ki/legis/num_act/pha2002243/)
        * [Public Holidays Act Consolidated](https://web.archive.org/web/20250404213705/https://www.paclii.org/ki/legis/consol_act/pho216.pdf)

    Note:
        The 2025 holidays list for Kiribati
        (http://web.archive.org/web/20250808191339/https://www.pso.gov.ki/kiribati-national-public-holidays-2025/)
        mentions the following holidays even though they do not appear in the law:
        Labor Day, National Culture and Senior Citizens Day, National Police Day, and World
        Teachers' Day.

        Additionally, the list shows that traditional holidays (International Women's Day,
        Labor Day, Gospel Day, and Human Rights and Peace Day) were celebrated on different
        dates than usual, seemingly shifting Thursday/Saturday holidays to the adjacent Friday.
        No legal provision or start year was cited for this adjustment.
    """

    country = "KI"
    observed_label = "%s (observed)"
    start_year = 1980

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        if self._year >= 2003:
            # International Women's Day.
            self._add_observed(self._add_womens_day("International Women's Day"))

        # Good Friday.
        self._add_good_friday("Good Friday")

        if self._year <= 2002:
            # Holy Saturday.
            self._add_holy_saturday("Holy Saturday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        if self._year >= 2003:
            # National Health Day.
            self._add_observed(self._add_holiday_apr_19("National Health Day"))

        if 1993 <= self._year <= 2002:
            # Public Holiday.
            self._add_observed(self._add_holiday_may_9("Public Holiday"))

        if self._year >= 2002:
            # Gospel Day.
            self._add_holiday_jul_11("Gospel Day")

        if self._year >= 1993:
            # National Day - Independence Anniversary.
            self._add_observed(self._add_holiday_jul_12("National Day - Independence Anniversary"))

        if self._year >= 2002:
            # National Day (in honor of Unimwane).
            self._add_holiday_jul_15("National Day (in honor of Unimwane)")
            # National Day (in honor of Unaine).
            self._add_observed(self._add_holiday_jul_16("National Day (in honor of Unaine)"))

        if self._year <= 2002:
            # Public Holiday.
            self._add_holiday_1st_mon_of_aug("Public Holiday")
        else:
            # Youth Day.
            self._add_observed(self._add_holiday_aug_5("Youth Day"))

        if self._year <= 1992:
            # Public Holiday.
            self._add_observed(self._add_holiday_nov_10("Public Holiday"))

        if self._year >= 1993:
            # Human Rights and Peace Day.
            name = "Human Rights and Peace Day"
            self._add_observed(
                self._add_holiday_dec_9(name)
                if self._year >= 2003
                else self._add_holiday_dec_10(name)
            )

        # Christmas Day.
        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two("Boxing Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)


class KI(Kiribati):
    pass


class KIR(Kiribati):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, MAR, JUN, JUL, AUG, SEP, OCT, NOV, DEC, FRI, SAT
from holidays.constants import CHRISTIAN, HEBREW, PUBLIC, SABIAN, YAZIDI
from holidays.groups import (
    ChristianHolidays,
    HebrewCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    MandaeanHolidays,
)
from holidays.holiday_base import HolidayBase


class Iraq(
    HolidayBase,
    ChristianHolidays,
    HebrewCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    MandaeanHolidays,
):
    """Iraq holidays.

    References:
        - [Law No. 10 of 1963](https://web.archive.org/web/20200220135833/http://wiki.dorar-aliraq.net:80/iraqilaws/law/1447.html)
        - [1968 Amendment to Law No. 10 of 1963](https://web.archive.org/web/20250730075454/https://wiki.dorar-aliraq.net/iraqilaws/law/18741.html)
        - [Law No. 110 of 1972](https://web.archive.org/web/20240820010527/https://wiki.dorar-aliraq.net/iraqilaws/law/5663.html)
        - [1973 Amendment to Law No. 110 of 1972](https://web.archive.org/web/20250730080337/https://wiki.dorar-aliraq.net/iraqilaws/law/6165.html)
        - [Official Holiday Law 2024](https://web.archive.org/web/20250629123214/https://natlex.ilo.org/dyn/natlex2/natlex2/files/download/116577/قانون%20العطل%20الرسمية%20-%20Copy%20-%20Copy_396.pdf)
    """

    country = "IQ"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (المقدرة)")
    start_year = 1964
    supported_categories = (CHRISTIAN, HEBREW, PUBLIC, SABIAN, YAZIDI)
    supported_languages = ("ar", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        HebrewCalendarHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=IraqIslamicHolidays, show_estimated=islamic_show_estimated
        )
        MandaeanHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year >= 1973:
            # New Year's Day.
            self._add_new_years_day(tr("رأس السنة الميلادية"))

        # Army Day.
        self._add_holiday_jan_6(tr("عيد الجيش"))

        if self._year <= 2024:
            # February 8 Revolution.
            self._add_holiday_feb_8(tr("ثورة 8 شباط"))

        if self._year >= 2025:
            # Commemoration of the Saddam Baath crimes against the Iraqi people.
            self._add_holiday_mar_16(tr("ذكرى جرائم البعث والأنفال والهجوم على حلبجة"))

        if self._year >= 1969:
            # Nowruz.
            self._add_holiday_mar_21(tr("عيد نوروز"))

        # Labor Day.
        self._add_labor_day(tr("عيد العمال العالمي"))

        if self._year <= 2023:
            # July 14 Revolution.
            self._add_holiday_jul_14(tr("ثورة 14 تموز"))

        if 1969 <= self._year <= 2023:
            # July 17 Revolution.
            self._add_holiday_jul_17(tr("ثورة 17 تموز"))

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Ashura.
        self._add_ashura_day(tr("عاشوراء"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("المولد النبوي الشريف"))

        # Eid al-Fitr.
        name = tr("عيد الفطر")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Day of Arafah.
        self._add_arafah_day(tr("يوم عرفة"))

        # Eid al-Adha.
        name = tr("عيد الأضحى")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)

        if self._year >= 2024:
            # Eid al-Ghadir.
            self._add_eid_al_ghadir_day(tr("عيد الغدير"))

    def _populate_christian_holidays(self):
        if self._year <= 1972:
            # New Year's Day.
            self._add_new_years_day(tr("رأس السنة الميلادية"))

        # Easter Sunday.
        self._add_easter_sunday(tr("أحد الفصح"))

        # Easter Monday.
        self._add_easter_monday(tr("إثنين الفصح"))

        # Christmas Day.
        self._add_christmas_day(tr("عيد الميلاد"))

    def _populate_hebrew_holidays(self):
        if self._year <= 2023:
            # Pesach.
            self._add_passover(tr("عيد الفصح"), range(2))

            # Yom Kippur.
            self._add_yom_kippur(tr("يوم الكفارة"))

            # Sukkot.
            self._add_sukkot(tr("عيد المظلة"), range(2))

    def _populate_sabian_holidays(self):
        if self._year <= 1972:
            return None

        # Great Feast.
        name = tr("يوما عيد البنجة")
        self._add_great_feast_day(name)
        self._add_great_feast_day_two(name)
        if self._year >= 2024:
            self._add_great_feast_day_three(name)
            self._add_great_feast_day_four(name)

        # Feast of Creation.
        name = tr("عيد الخليقة")
        self._add_parwanaya_day(name)
        self._add_parwanaya_day_two(name)
        if self._year >= 2024:
            self._add_parwanaya_day_three(name)
            self._add_parwanaya_day_four(name)
            self._add_parwanaya_day_five(name)

        # Little Feast.
        name = tr("عيد الصغير")
        self._add_dehwa_hanina_day(name)
        if self._year >= 2024:
            self._add_dehwa_hanina_day_two(name)

        # Prophet Yahya's Birthday.
        self._add_dehwa_daimana_day(tr("مولد النبي يحيى عليه السلام"))

    def _populate_yazidi_holidays(self):
        # Yazidi New Year.
        self._add_holiday_1st_wed_from_apr_14(tr("رأس السنة الإيزيدية"))

        # Yazidi Summer Festival.
        name = tr("مهرجان الصيف اليزيدي")
        if self._year <= 2023:
            self._add_holiday_jul_31(name)
            self._add_holiday_aug_1(name)
        self._add_holiday_aug_2(name)
        self._add_holiday_aug_3(name)

        # Feast of the Assembly.
        name = tr("عيد الجمعية")
        self._add_holiday_oct_6(name)
        self._add_holiday_oct_7(name)
        self._add_holiday_oct_8(name)
        self._add_holiday_oct_9(name)
        self._add_holiday_oct_10(name)
        self._add_holiday_oct_11(name)
        self._add_holiday_oct_12(name)
        self._add_holiday_oct_13(name)

        # Feast of Êzî.
        self._add_holiday_1st_fri_from_dec_14(tr("عيد إيزي"))


class IQ(Iraq):
    pass


class IRQ(Iraq):
    pass


class IraqIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20240908214306/https://www.timeanddate.com/holidays/iraq/ashura
    ASHURA_DATES_CONFIRMED_YEARS = (2013, 2025)
    ASHURA_DATES = {
        2013: (NOV, 14),
        2015: (OCT, 24),
        2016: (OCT, 12),
        2017: (OCT, 1),
        2019: (SEP, 10),
        2020: (AUG, 30),
        2021: (AUG, 19),
        2023: (JUL, 29),
    }

    # https://web.archive.org/web/20250524093010/https://www.timeanddate.com/holidays/iraq/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2013, 2025)
    EID_AL_ADHA_DATES = {
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
        2019: (AUG, 12),
    }

    # https://web.archive.org/web/20250514061230/https://www.timeanddate.com/holidays/iraq/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2013, 2025)
    EID_AL_FITR_DATES = {
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
        2025: (MAR, 31),
    }

    # https://web.archive.org/web/20241009070642/https://www.timeanddate.com/holidays/iraq/eid-al-ghadeer
    EID_AL_GHADIR_DATES_CONFIRMED_YEARS = (2024, 2025)
    EID_AL_GHADIR_DATES = {
        2025: (JUN, 15),
    }

    # https://web.archive.org/web/20240720191242/https://www.timeanddate.com/holidays/iraq/islamic-new-year
    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (2013, 2025)
    HIJRI_NEW_YEAR_DATES = {
        2013: (NOV, 5),
        2015: (OCT, 15),
        2016: (OCT, 3),
        2017: (SEP, 22),
        2020: (AUG, 21),
    }

    # https://web.archive.org/web/20240918050148/https://www.timeanddate.com/holidays/iraq/prophet-birthday
    MAWLID_DATES_CONFIRMED_YEARS = (2013, 2025)
    MAWLID_DATES = {
        2014: (JAN, 14),
        2016: (DEC, 12),
        2017: (DEC, 1),
        2018: (NOV, 21),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import HALF_DAY, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class FaroeIslands(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Faroe Islands holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Faroe_Islands>
        * [Decree no. 52 of 23 April 2003](https://web.archive.org/web/20241204233745/https://logir.fo/Kunngerd/52-fra-23-04-2003-um-flagging-fra-landsins-bygningum)
        * [2007](https://web.archive.org/web/20250125193236/https://www.framtak.com/info/holidays.html))
        * [2024](https://web.archive.org/web/20250615193008/https://visitfaroeislands.com/en/plan-your-stay/get-ready-for-your-trip/general-facts/public-holidays)
        * [2025](https://web.archive.org/web/20250615193215/https://guidetofaroeislands.fo/travel-information/faroe-islands-holiday/)
    """

    country = "FO"
    default_language = "fo"
    # Denmark granted home rule to the Faroe Islands on 30 March 1948.
    start_year = 1949
    supported_categories = (HALF_DAY, PUBLIC)
    supported_languages = ("da", "en_US", "fo", "is", "no", "sv")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Nýggjársdagur"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Skírhósdagur"))

        # Good Friday.
        self._add_good_friday(tr("Langifríggjadagur"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Páskadagur"))

        # Easter Monday.
        self._add_easter_monday(tr("Annar páskadagur"))

        # Great Prayer Day.
        self._add_holiday_26_days_past_easter(tr("Dýri biðidagur"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Kristi himmalsferðardagur"))

        # Whit Sunday.
        self._add_whit_sunday(tr("Hvítusunnudagur"))

        # Whit Monday.
        self._add_whit_monday(tr("Annar hvítusunnudagur"))

        # Saint Olaf's Day.
        self._add_holiday_jul_29(tr("Ólavsøkudagur"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Jólaaftan"))

        # Christmas Day.
        self._add_christmas_day(tr("Jóladagur"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Annar jóladagur"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Nýggjársaftan"))

    def _populate_half_day_holidays(self):
        # National Flag Day.
        self._add_holiday_apr_25(tr("Flaggdagur"))

        # Constitution Day.
        self._add_holiday_jun_5(tr("Grundlógardagur"))

        # Saint Olaf's Eve.
        self._add_holiday_jul_28(tr("Ólavsøkuaftan"))


class FO(FaroeIslands):
    pass


class FRO(FaroeIslands):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.gregorian import (
    JAN,
    FEB,
    MAR,
    APR,
    MAY,
    JUN,
    JUL,
    AUG,
    SEP,
    OCT,
    NOV,
    DEC,
    SAT,
    SUN,
    _timedelta,
)
from holidays.constants import HALF_DAY, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_TO_PREV_FRI, SUN_TO_NEXT_MON


class NewYorkStockExchange(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays
):
    """New York Stock Exchange holidays.

    References:
        * [NYSE Rules](https://web.archive.org/web/20240118104341/https://www.nyse.com/publicdocs/nyse/regulation/nyse/NYSE_Rules.pdf)
        * [NYSE History - Timeline](https://web.archive.org/web/20150113014537/https://www.nyse.com/about/history/timeline_trading.html)

    Historical data:
        * [History of NYSE Holidays](https://web.archive.org/web/20221206064307/https://s3.amazonaws.com/armstrongeconomics-wp/2013/07/NYSE-Closings.pdf)
        * [NYSE Holidays & Trading Hours](https://web.archive.org/web/20211101162021/https://www.nyse.com/markets/hours-calendars)
    """

    market = "XNYS"
    observed_label = "%s (observed)"
    start_year = 1863
    supported_categories = (HALF_DAY, PUBLIC)

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, NewYorkStockExchangeStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_TO_PREV_FRI + SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _add_observed(self, dt: date, **kwargs) -> tuple[bool, date | None]:
        kwargs["rule"] = SUN_TO_NEXT_MON if dt.year <= 1952 else self._observed_rule
        return super()._add_observed(dt, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # September 29, 1952: The NYSE changes its trading hours
        # to 10am to 3:30pm Monday-Friday, and closes on Saturdays.
        return {SAT, SUN} if dt >= date(1952, SEP, 29) else {SUN}

    def _populate_ranged_holidays(
        self, name: str, start: tuple[int, int], end: tuple[int, int], step: int = 1
    ):
        start_date = date(self._year, *start)
        end_date = date(self._year, *end)
        for dt in (
            _timedelta(start_date, n) for n in range(0, (end_date - start_date).days + 1, step)
        ):
            if self._is_weekend(dt) or dt in self:
                continue
            self._add_holiday(name, dt)

    def _populate_public_holidays(self):
        # According to NYSE Rules (Rule 51), when any holiday observed by the Exchange falls on
        # a Saturday, the Exchange will not be open for business on the preceding Friday ...,
        # unless unusual business conditions exist, such as the ending of a monthly or the yearly
        # accounting period.
        # Therefore, New Year's Day holiday is not moved to Dec 31, even after 1952 amendments.

        # New Year's Day.
        self._move_holiday(self._add_new_years_day("New Year's Day"))

        # Martin Luther King Jr. Day.
        if self._year >= 1998:
            self._add_holiday_3rd_mon_of_jan("Martin Luther King Jr. Day")

        # Lincoln's Birthday.
        if 1896 <= self._year <= 1953:
            self._move_holiday(self._add_holiday_feb_12("Lincoln's Birthday"))

        # Washington's Birthday.
        name = "Washington's Birthday"
        if self._year >= 1971:
            self._add_holiday_3rd_mon_of_feb(name)
        else:
            self._move_holiday(self._add_holiday_feb_22(name))

        # Good Friday.
        if self._year not in {1898, 1906, 1907}:
            self._add_good_friday("Good Friday")

        # Memorial Day.
        if self._year >= 1873:
            name = "Memorial Day"
            if self._year >= 1971:
                self._add_holiday_last_mon_of_may(name)
            else:
                self._move_holiday(self._add_holiday_may_30(name))

        # Flag Day.
        if 1916 <= self._year <= 1953:
            self._move_holiday(self._add_holiday_jun_14("Flag Day"))

        # Juneteenth National Independence Day.
        if self._year >= 2022:
            self._move_holiday(self._add_holiday_jun_19("Juneteenth National Independence Day"))

        # Independence Day.
        self._move_holiday(self._add_holiday_jul_4("Independence Day"))

        # Labor Day.
        if self._year >= 1887:
            self._add_holiday_1st_mon_of_sep("Labor Day")

        # Columbus Day.
        if 1909 <= self._year <= 1953:
            self._move_holiday(self._add_columbus_day("Columbus Day"))

        # Election Day.
        if self._year <= 1968 or self._year in {1972, 1976, 1980}:
            self._add_holiday_1_day_past_1st_mon_of_nov("Election Day")

        # Veteran's Day.
        if 1934 <= self._year <= 1953:
            self._move_holiday(self._add_remembrance_day("Veteran's Day"))

        # Thanksgiving Day.
        thanksgiving_day_dates = {
            1865: (DEC, 7),
            1869: (NOV, 18),
            1939: (NOV, 23),
            1940: (NOV, 21),
            1941: (NOV, 20),
        }
        name = "Thanksgiving Day"
        if dt := thanksgiving_day_dates.get(self._year):
            self._add_holiday(name, dt)
        elif self._year >= 1942:
            self._add_holiday_4th_thu_of_nov(name)
        else:
            self._add_holiday_last_thu_of_nov(name)

        # Christmas Day.
        self._move_holiday(self._add_christmas_day("Christmas Day"))

        # Special holidays.

        if self._year == 1914:
            self._populate_ranged_holidays("Pending outbreak of World War I", (JUL, 31), (NOV, 27))
        elif self._year == 1968:
            # WED.
            self._populate_ranged_holidays("Paperwork Crisis", (JUN, 12), (DEC, 24), 7)

        # Closed Saturdays.
        closed_sat_date_ranges = {
            # Start date, end date.
            1944: ((AUG, 19), (SEP, 2)),
            1945: ((JUL, 7), (SEP, 1)),
            1946: ((JUN, 1), (SEP, 28)),
            1947: ((MAY, 31), (SEP, 27)),
            1948: ((MAY, 29), (SEP, 25)),
            1949: ((MAY, 28), (SEP, 24)),
            1950: ((JUN, 3), (SEP, 30)),
            1951: ((JUN, 2), (SEP, 29)),
            1952: ((MAY, 31), (SEP, 27)),
        }
        if self._year in closed_sat_date_ranges:
            start, end = closed_sat_date_ranges[self._year]
            self._populate_ranged_holidays("Closed Saturday", start, end, 7)

    def _populate_half_day_holidays(self):
        # %s (markets close at 1:00pm).
        close_1pm_label = "%s (markets close at 1:00pm)"

        # %s (markets close at 2:00pm).
        close_2pm_label = "%s (markets close at 2:00pm)"

        # %s (markets close at 2:30pm).
        close_2_30pm_label = "%s (markets close at 2:30pm)"

        # %s (markets close at 3:00pm).
        close_3pm_label = "%s (markets close at 3:00pm)"

        # Special holidays.

        name_paperwork_crisis = "Paperwork Crisis"
        if self._year == 1968:
            self._populate_ranged_holidays(
                close_2pm_label % "Back office work load", (JAN, 22), (MAR, 1)
            )
        elif self._year == 1969:
            # THU.
            self._populate_ranged_holidays(
                close_2pm_label % name_paperwork_crisis, (JAN, 2), (JUL, 3), 7
            )

            # MON-FRI.
            self._populate_ranged_holidays(
                close_2_30pm_label % name_paperwork_crisis, (JUL, 7), (SEP, 26)
            )

            # MON-FRI.
            self._populate_ranged_holidays(
                close_3pm_label % name_paperwork_crisis, (SEP, 29), (DEC, 31)
            )
        elif self._year == 1970:
            # MON-FRI.
            self._populate_ranged_holidays(
                close_3pm_label % name_paperwork_crisis, (JAN, 2), (MAY, 1)
            )

        # 1990-1992 early closings are covered by special holidays.
        if self._year >= 1993:
            # Day before Independence Day.
            jul_4 = (JUL, 4)
            if (
                self._is_weekday(jul_4)
                and not self._is_monday(jul_4)
                and self._year not in {1996, 2002}
            ):
                self._add_holiday_jul_3(close_1pm_label % "Day before Independence Day")

            # Day after Thanksgiving Day.
            self._add_holiday_1_day_past_4th_thu_of_nov(
                close_1pm_label % "Day after Thanksgiving Day"
            )

            # Christmas Eve.
            if self._is_weekday(self._christmas_day) and not self._is_monday(self._christmas_day):
                self._add_christmas_eve(close_1pm_label % "Christmas Eve")


class XNYS(NewYorkStockExchange):
    pass


class NYSE(NewYorkStockExchange):
    pass


class NewYorkStockExchangeStaticHolidays:
    """New York Stock Exchange special holidays.

    References:
        * <https://web.archive.org/web/20250421093104/https://guides.loc.gov/presidents-portraits/chronological>
        * <https://web.archive.org/web/20250208135423/https://www.presidency.ucsb.edu/documents/proclamation-3561-national-day-mourning-for-president-kennedy>
    """

    # %s (markets close at 11:00am).
    close_11am_label = "%s (markets close at 11:00am)"

    # %s (markets close at 12:00pm).
    close_12pm_label = "%s (markets close at 12:00pm)"

    # %s (markets close at 12:30pm).
    close_12_30pm_label = "%s (markets close at 12:30pm)"

    # %s (markets close at 1:00pm).
    close_1pm_label = "%s (markets close at 1:00pm)"

    # %s (markets close at 2:00pm).
    close_2pm_label = "%s (markets close at 2:00pm)"

    # %s (markets close at 2:30pm).
    close_2_30pm_label = "%s (markets close at 2:30pm)"

    # %s (markets close at 3:00pm).
    close_3pm_label = "%s (markets close at 3:00pm)"

    # %s (markets close at 3:30pm).
    close_3_30pm_label = "%s (markets close at 3:30pm)"

    # Blizzard of 1888.
    name_blizzard_of_1888 = "Blizzard of 1888"

    # Centennial of George Washington's Inauguration.
    name_centennial_of_gw_inauguration = "Centennial of George Washington's Inauguration"

    # Columbian Celebration.
    name_columbian_celebration = "Columbian Celebration"

    # Heatless Day.
    name_heatless_day = "Heatless Day"

    # Catch Up Day.
    name_catch_up_day = "Catch Up Day"

    # Special Bank Holiday.
    name_national_banking_holiday = "National Banking Holiday"

    # V-J Day. End of World War II.
    name_vj_day = "V-J Day. End of World War II"

    # Christmas Eve.
    name_christmas_eve = "Christmas Eve"

    # Closed following Attacks on the World Trade Center.
    name_closed_following_wtc_attacks = "Closed following Attacks on the World Trade Center"

    # Hurricane Sandy.
    name_hurricane_sandy = "Hurricane Sandy"

    # Liberty Day.
    name_liberty_day = "Liberty Day"

    # Heavy volume. To allow member firm offices to catch up on work.
    name_heavy_volume_catch_up = "Heavy volume. To allow member firm offices to catch up on work"

    # Saturday after Washington's Birthday.
    name_saturday_after_washingtons_birthday = "Saturday after Washington's Birthday"

    # Saturday after Good Friday.
    name_saturday_after_good_friday = "Saturday after Good Friday"

    # Saturday before Decoration Day.
    name_saturday_before_decoration_day = "Saturday before Decoration Day"

    # Saturday after Decoration Day.
    name_saturday_after_decoration_day = "Saturday after Decoration Day"

    # Saturday before Independence Day.
    name_saturday_before_independence_day = "Saturday before Independence Day"

    # Saturday after Independence Day.
    name_saturday_after_independence_day = "Saturday after Independence Day"

    # Day after Independence Day.
    name_day_after_independence_day = "Day after Independence Day"

    # Saturday before Labor Day.
    name_saturday_before_labor_day = "Saturday before Labor Day"

    # Saturday after Columbus Day.
    name_saturday_after_columbus_day = "Saturday after Columbus Day"

    # Saturday before Christmas.
    name_saturday_before_christmas_day = "Saturday before Christmas Day"

    # Friday after Christmas Day.
    name_friday_after_christmas = "Friday after Christmas Day"

    # Saturday after Christmas Day.
    name_saturday_after_christmas_day = "Saturday after Christmas Day"

    # Admiral Dewey Celebration.
    name_admiral_dewey_celebration = "Admiral Dewey Celebration"

    # Draft Registration Day.
    name_draft_registration_day = "Draft Registration Day"

    # To allow offices to catch up on work.
    name_to_catch_up = "To allow offices to catch up on work"

    # Volume activity.
    name_volume_activity = "Volume activity"

    # Transit strike.
    name_transit_strike = "Transit strike"

    # Back office work load.
    name_back_office_work_load = "Back office work load"

    # Snowstorm.
    name_snowstorm = "Snowstorm"

    # Shortened hours following market break.
    name_shortened_hours_following_market_break = "Shortened hours following market break"

    special_public_holidays = {
        1885: (AUG, 8, "Funeral of former President Ulysses S. Grant"),
        1887: (
            (JUL, 2, name_saturday_before_independence_day),
            (DEC, 24, name_saturday_before_christmas_day),
        ),
        1888: (
            (MAR, 12, name_blizzard_of_1888),
            (MAR, 13, name_blizzard_of_1888),
            (SEP, 1, name_saturday_before_labor_day),
            (NOV, 30, "Friday after Thanksgiving Day"),
        ),
        1889: (
            (APR, 29, name_centennial_of_gw_inauguration),
            (APR, 30, name_centennial_of_gw_inauguration),
            (MAY, 1, name_centennial_of_gw_inauguration),
        ),
        1890: (JUL, 5, name_saturday_after_independence_day),
        1891: (DEC, 26, name_saturday_after_christmas_day),
        1892: (
            (JUL, 2, name_saturday_before_independence_day),
            (OCT, 12, name_columbian_celebration),
            (OCT, 21, name_columbian_celebration),
            (OCT, 22, name_columbian_celebration),
        ),
        1893: (APR, 27, name_columbian_celebration),
        1896: (DEC, 26, name_saturday_after_christmas_day),
        1897: (APR, 27, "Grant's Birthday"),
        1898: (
            (MAY, 4, "Charter Day"),
            (JUL, 2, name_saturday_before_independence_day),
            (AUG, 20, "Welcome of naval commanders"),
            (SEP, 3, name_saturday_before_labor_day),
            (DEC, 24, name_saturday_before_christmas_day),
        ),
        1899: (
            (FEB, 11, "Saturday before Lincoln's Birthday"),
            (MAY, 29, "Monday before Decoration Day"),
            (JUL, 3, "Monday before Independence Day"),
            (SEP, 29, name_admiral_dewey_celebration),
            (SEP, 30, name_admiral_dewey_celebration),
            (NOV, 25, "Funeral of Vice-President Garret A. Hobart"),
        ),
        1900: (
            (APR, 14, name_saturday_after_good_friday),
            (SEP, 1, name_saturday_before_labor_day),
            (DEC, 24, name_saturday_before_christmas_day),
        ),
        1901: (
            (FEB, 2, "Funeral of Queen Victoria of England"),
            (FEB, 23, name_saturday_after_washingtons_birthday),
            (APR, 6, name_saturday_after_good_friday),
            (APR, 27, "Moved to temporary quarters in Produce Exchange"),
            (MAY, 11, "Enlarged temporary quarters in Produce Exchange"),
            (JUL, 5, "Friday after Independence Day"),
            (JUL, 6, name_saturday_after_independence_day),
            (AUG, 31, name_saturday_before_labor_day),
            (SEP, 14, "Death of President William McKinley"),
            (SEP, 19, "Funeral of President William McKinley"),
        ),
        1902: (
            (MAR, 29, name_saturday_after_good_friday),
            (MAY, 31, name_saturday_after_decoration_day),
            (JUL, 5, name_saturday_after_independence_day),
            (AUG, 9, "Coronation of King Edward VII of England"),
            (AUG, 30, name_saturday_before_labor_day),
        ),
        1903: (
            (FEB, 21, "Saturday before Washington's Birthday"),
            (APR, 11, name_saturday_after_good_friday),
            (APR, 22, "Opening of new NYSE building"),
            (SEP, 5, name_saturday_before_labor_day),
            (DEC, 26, name_saturday_after_christmas_day),
        ),
        1904: (
            (MAY, 28, name_saturday_before_decoration_day),
            (JUL, 2, name_saturday_before_independence_day),
            (SEP, 3, name_saturday_before_labor_day),
            (DEC, 24, name_saturday_before_christmas_day),
        ),
        1905: (APR, 22, name_saturday_after_good_friday),
        1907: (
            (FEB, 23, name_saturday_after_washingtons_birthday),
            (MAR, 30, name_saturday_after_good_friday),
            (AUG, 31, name_saturday_before_labor_day),
        ),
        1908: (
            (APR, 18, name_saturday_after_good_friday),
            (SEP, 5, name_saturday_before_labor_day),
            (DEC, 26, name_saturday_after_christmas_day),
        ),
        1909: (
            (FEB, 13, "Saturday after Lincoln's Birthday"),
            (APR, 10, name_saturday_after_good_friday),
            (MAY, 29, name_saturday_before_decoration_day),
            (JUL, 3, name_saturday_before_independence_day),
            (SEP, 4, name_saturday_before_labor_day),
            (SEP, 25, "Reception Day of the Hudson-Fulton Celebration"),
        ),
        1910: (
            (MAR, 26, name_saturday_after_good_friday),
            (MAY, 28, name_saturday_before_decoration_day),
            (JUL, 2, name_saturday_before_independence_day),
            (SEP, 3, name_saturday_before_labor_day),
            (DEC, 24, name_saturday_before_christmas_day),
        ),
        1911: (
            (APR, 15, name_saturday_after_good_friday),
            (SEP, 2, name_saturday_before_labor_day),
            (DEC, 23, name_saturday_before_christmas_day),
        ),
        1912: (
            (AUG, 31, name_saturday_before_labor_day),
            (NOV, 2, "Funeral of Vice-President James S. Sherman"),
        ),
        1913: (
            (MAR, 22, name_saturday_after_good_friday),
            (MAY, 31, name_saturday_after_decoration_day),
            (JUL, 5, name_saturday_after_independence_day),
            (AUG, 30, name_saturday_before_labor_day),
        ),
        1916: (DEC, 30, "Saturday before New Year's Day"),
        1917: (
            (JUN, 5, name_draft_registration_day),
            (AUG, 4, "Heat"),
            (SEP, 1, name_saturday_before_labor_day),
            (OCT, 13, name_saturday_after_columbus_day),
        ),
        1918: (
            (JAN, 28, name_heatless_day),
            (FEB, 4, name_heatless_day),
            (FEB, 11, name_heatless_day),
            (SEP, 12, name_draft_registration_day),
            (NOV, 11, "Armistice signed"),
        ),
        1919: (
            (MAR, 25, "Homecoming of 27th Division"),
            (MAY, 6, "Parade of 77th Division"),
            (MAY, 31, name_saturday_after_decoration_day),
            (JUL, 5, name_saturday_after_independence_day),
            (JUL, 19, "Heat and to allow offices to catch up on work"),
            (AUG, 2, name_to_catch_up),
            (AUG, 16, name_to_catch_up),
            (AUG, 30, name_saturday_before_labor_day),
            (SEP, 10, "Return of General John J. Pershing"),
        ),
        1920: (
            (APR, 3, name_saturday_after_good_friday),
            (MAY, 1, "Many firms changed office locations"),
            (JUL, 3, name_saturday_before_independence_day),
            (SEP, 4, name_saturday_before_labor_day),
        ),
        1921: (
            (MAY, 28, name_saturday_before_decoration_day),
            (JUL, 2, name_saturday_before_independence_day),
            (SEP, 3, name_saturday_before_labor_day),
            (NOV, 11, "Veteran's Day"),
        ),
        1922: (DEC, 23, name_saturday_before_christmas_day),
        1923: (
            (AUG, 3, "Death of President Warren G. Harding"),
            (AUG, 10, "Funeral of President Warren G. Harding"),
        ),
        1924: (MAY, 31, name_saturday_after_decoration_day),
        1925: (DEC, 26, name_saturday_after_christmas_day),
        1926: (
            (MAY, 29, name_saturday_before_decoration_day),
            (JUL, 3, name_saturday_before_independence_day),
            (SEP, 4, name_saturday_before_labor_day),
        ),
        1927: (JUN, 13, "Parade for Colonel Charles A. Lindbergh"),
        1928: (
            (APR, 7, name_heavy_volume_catch_up),
            (APR, 21, name_heavy_volume_catch_up),
            (MAY, 5, name_heavy_volume_catch_up),
            (MAY, 12, name_heavy_volume_catch_up),
            (MAY, 19, name_heavy_volume_catch_up),
            (MAY, 26, name_heavy_volume_catch_up),
            (NOV, 24, name_heavy_volume_catch_up),
        ),
        1929: (
            (FEB, 9, name_heavy_volume_catch_up),
            (FEB, 23, name_saturday_after_washingtons_birthday),
            (MAR, 30, name_saturday_after_good_friday),
            (AUG, 31, name_saturday_before_labor_day),
            (NOV, 1, name_catch_up_day),
            (NOV, 2, name_catch_up_day),
            (NOV, 9, name_catch_up_day),
            (NOV, 16, name_catch_up_day),
            (NOV, 23, name_catch_up_day),
            (NOV, 29, name_catch_up_day),
            (NOV, 30, name_catch_up_day),
        ),
        1930: (
            (APR, 19, name_saturday_after_good_friday),
            (MAY, 31, name_saturday_after_decoration_day),
            (JUL, 5, name_saturday_after_independence_day),
            (AUG, 30, name_saturday_before_labor_day),
        ),
        1931: (
            (SEP, 5, name_saturday_before_labor_day),
            (DEC, 26, name_saturday_after_christmas_day),
        ),
        1932: (JUL, 2, name_saturday_before_independence_day),
        1933: (
            (JAN, 7, "Funeral of former President Calvin Coolidge"),
            (MAR, 4, "State Banking Holiday"),
            (MAR, 6, name_national_banking_holiday),
            (MAR, 7, name_national_banking_holiday),
            (MAR, 8, name_national_banking_holiday),
            (MAR, 9, name_national_banking_holiday),
            (MAR, 10, name_national_banking_holiday),
            (MAR, 11, name_national_banking_holiday),
            (MAR, 13, name_national_banking_holiday),
            (MAR, 14, name_national_banking_holiday),
            (JUL, 28, name_volume_activity),
            (AUG, 5, name_volume_activity),
            (AUG, 12, name_volume_activity),
            (AUG, 19, name_volume_activity),
            (AUG, 26, name_volume_activity),
            (SEP, 2, name_volume_activity),
        ),
        1936: (DEC, 26, name_saturday_after_christmas_day),
        1937: (
            (MAY, 29, name_saturday_before_decoration_day),
            (JUL, 3, name_saturday_before_independence_day),
        ),
        1945: (
            (APR, 14, "National Day of Mourning for President Franklin D. Roosevelt"),
            (AUG, 15, name_vj_day),
            (AUG, 16, name_vj_day),
            (OCT, 13, name_saturday_after_columbus_day),
            (OCT, 27, "Navy Day"),
            (DEC, 24, name_christmas_eve),
        ),
        1946: (
            (FEB, 23, name_saturday_after_washingtons_birthday),
            (MAY, 25, "Railroad strike"),
        ),
        1948: (JAN, 3, "Severe weather conditions"),
        1949: (DEC, 24, name_christmas_eve),
        1950: (DEC, 23, "Saturday before Christmas Eve"),
        1954: (DEC, 24, name_christmas_eve),
        1956: (DEC, 24, name_christmas_eve),
        1958: (DEC, 26, "Day after Christmas"),
        1961: (MAY, 29, "Day before Decoration Day"),
        1963: (NOV, 25, "Funeral of President John F. Kennedy"),
        1965: (DEC, 24, name_christmas_eve),
        1968: (
            (FEB, 12, "Lincoln's Birthday"),
            (APR, 9, "National Day of Mourning for Martin Luther King, Jr."),
            (JUL, 5, name_day_after_independence_day),
        ),
        1969: (
            (FEB, 10, "Heavy Snow"),
            (MAR, 31, "Funeral of former President Dwight D. Eisenhower"),
            (JUL, 21, "National Day of Participation for the Lunar Exploration"),
        ),
        1972: (DEC, 28, "Funeral of former President Harry S. Truman"),
        1973: (JAN, 25, "Funeral of former President Lyndon B. Johnson"),
        1977: (JUL, 14, "Blackout in New York City"),
        1985: (SEP, 27, "Hurricane Gloria"),
        1994: (APR, 27, "Funeral of former President Richard M. Nixon"),
        2001: (
            (SEP, 11, name_closed_following_wtc_attacks),
            (SEP, 12, name_closed_following_wtc_attacks),
            (SEP, 13, name_closed_following_wtc_attacks),
            (SEP, 14, name_closed_following_wtc_attacks),
        ),
        2004: (JUN, 11, "National Day of Mourning for former President Ronald Reagan"),
        2007: (JAN, 2, "National Day of Mourning for former President Gerald R. Ford"),
        2012: (
            (OCT, 29, name_hurricane_sandy),
            (OCT, 30, name_hurricane_sandy),
        ),
        2018: (DEC, 5, "National Day of Mourning for former President George H. W. Bush"),
        2025: (JAN, 9, "National Day of Mourning for former President Jimmy Carter"),
    }

    # Late opens and partial suspensions of work during the day are not implemented, such as:
    # * May 20, 1910 (Fri) - Closed from 10:00am to 12:00pm. Funeral of King Edward VII.
    # * Apr. 14, 1913 (Mon) - Closed from 10:00am to 12:00pm. Funeral of J. P. Morgan.
    # * Sept. 22, 1913 (Mon) - Closed from 10:00am to 12:00pm. Funeral of
    #   Mayor William J. Gaynor.
    # * July 11, 1918 (Thu) - Closed from 11:00am to 12:00pm. Funeral of former Mayor
    #   John Purroy Mitchell.
    # * Aug. 8, 1923 (Wed) - Closed from 11:00am to 12:30pm during funeral services for President
    #   Warren G. Harding in Washington, DC.
    # * Jan. 28, 1936 (Tue) - Closed from 10:00 to 11:00am. Funeral of King George V of England.
    # * May 18, 1942 (Mon) - Closed from 12:00pm to 1:00pm. NYSE 150th anniversary.
    # * June 19, 1945 (Tue) - Closed from 11:00am to 1:00pm. Parade for General Eisenhower.
    # * Apr. 20, 1951 (Fri) - Closed from 11:00am to 1:00pm. Parade for General MacArthur.

    special_half_day_holidays = {
        1908: (JUN, 26, close_1pm_label % "Funeral of former President Grover Cleveland"),
        1910: (MAY, 7, close_11am_label % "Death of King Edward VII of England"),
        1917: (
            (AUG, 29, close_12pm_label % "Parade of National Guard"),
            (OCT, 24, close_12pm_label % name_liberty_day),
        ),
        1918: (
            (APR, 26, close_12pm_label % name_liberty_day),
            (NOV, 7, close_2_30pm_label % "False armistice report"),
        ),
        1919: (JAN, 7, close_12_30pm_label % "Funeral of former President Theodore Roosevelt"),
        1920: (SEP, 16, close_12pm_label % "Wall Street explosion"),
        1924: (FEB, 6, close_12_30pm_label % "Funeral of former President Woodrow Wilson"),
        1925: (
            SEP,
            18,
            close_2_30pm_label % "Funeral of former NYSE president Seymour L. Cromwell",
        ),
        1928: (
            (MAY, 21, close_2pm_label % name_heavy_volume_catch_up),
            (MAY, 22, close_2pm_label % name_heavy_volume_catch_up),
            (MAY, 23, close_2pm_label % name_heavy_volume_catch_up),
            (MAY, 24, close_2pm_label % name_heavy_volume_catch_up),
            (MAY, 25, close_2pm_label % name_heavy_volume_catch_up),
        ),
        1929: (
            (NOV, 6, close_1pm_label % name_catch_up_day),
            (NOV, 7, close_1pm_label % name_catch_up_day),
            (NOV, 8, close_1pm_label % name_catch_up_day),
            (NOV, 11, close_1pm_label % name_catch_up_day),
            (NOV, 12, close_1pm_label % name_catch_up_day),
            (NOV, 13, close_1pm_label % name_catch_up_day),
            (NOV, 14, close_1pm_label % name_catch_up_day),
            (NOV, 15, close_1pm_label % name_catch_up_day),
            (NOV, 18, close_1pm_label % name_catch_up_day),
            (NOV, 19, close_1pm_label % name_catch_up_day),
            (NOV, 20, close_1pm_label % name_catch_up_day),
            (NOV, 21, close_1pm_label % name_catch_up_day),
            (NOV, 22, close_1pm_label % name_catch_up_day),
        ),
        1930: (MAR, 11, close_12_30pm_label % "Funeral of former President William Howard Taft"),
        1933: (SEP, 13, close_12pm_label % "NRA demonstration"),
        1951: (DEC, 24, close_1pm_label % name_christmas_eve),
        1963: (NOV, 22, "Assassination of President John F. Kennedy (markets close at 2:07pm)"),
        1964: (OCT, 23, close_2pm_label % "Funeral of former President Herbert C. Hoover"),
        1966: (
            (JAN, 6, close_2pm_label % name_transit_strike),
            (JAN, 7, close_2pm_label % name_transit_strike),
            (JAN, 10, close_2pm_label % name_transit_strike),
            (JAN, 11, close_2pm_label % name_transit_strike),
            (JAN, 12, close_2pm_label % name_transit_strike),
            (JAN, 13, close_2pm_label % name_transit_strike),
            (JAN, 14, close_2pm_label % name_transit_strike),
        ),
        1967: (
            (AUG, 8, close_2pm_label % name_back_office_work_load),
            (AUG, 9, close_2pm_label % name_back_office_work_load),
            (AUG, 10, close_2pm_label % name_back_office_work_load),
            (AUG, 11, close_2pm_label % name_back_office_work_load),
            (AUG, 14, close_2pm_label % name_back_office_work_load),
            (AUG, 15, close_2pm_label % name_back_office_work_load),
            (AUG, 16, close_2pm_label % name_back_office_work_load),
            (AUG, 17, close_2pm_label % name_back_office_work_load),
            (AUG, 18, close_2pm_label % name_back_office_work_load),
        ),
        1974: (DEC, 24, close_2pm_label % name_christmas_eve),
        1975: (
            (FEB, 12, close_2_30pm_label % name_snowstorm),
            (DEC, 24, close_2pm_label % name_christmas_eve),
        ),
        1976: (AUG, 9, close_3pm_label % "Hurricane watch"),
        1978: (FEB, 6, close_2pm_label % name_snowstorm),
        1981: (
            (MAR, 30, "Assassination attempt on President Reagan (markets close at 3:17pm)"),
            (SEP, 9, "Con Edison power failure in lower Manhattan (markets close at 3:28pm)"),
        ),
        1987: (
            (OCT, 23, close_2pm_label % name_shortened_hours_following_market_break),
            (OCT, 26, close_2pm_label % name_shortened_hours_following_market_break),
            (OCT, 27, close_2pm_label % name_shortened_hours_following_market_break),
            (OCT, 28, close_2pm_label % name_shortened_hours_following_market_break),
            (OCT, 29, close_2pm_label % name_shortened_hours_following_market_break),
            (OCT, 30, close_2pm_label % name_shortened_hours_following_market_break),
            (NOV, 2, close_2_30pm_label % name_shortened_hours_following_market_break),
            (NOV, 3, close_2_30pm_label % name_shortened_hours_following_market_break),
            (NOV, 4, close_2_30pm_label % name_shortened_hours_following_market_break),
            (NOV, 5, close_3pm_label % name_shortened_hours_following_market_break),
            (NOV, 6, close_3pm_label % name_shortened_hours_following_market_break),
            (NOV, 9, close_3_30pm_label % name_shortened_hours_following_market_break),
            (NOV, 10, close_3_30pm_label % name_shortened_hours_following_market_break),
            (NOV, 11, close_3_30pm_label % name_shortened_hours_following_market_break),
        ),
        1990: (DEC, 24, close_2pm_label % name_christmas_eve),
        1991: (DEC, 24, close_2pm_label % name_christmas_eve),
        1992: (
            (NOV, 27, close_2pm_label % "Day after Thanksgiving Day"),
            (DEC, 24, close_2pm_label % name_christmas_eve),
        ),
        1994: (FEB, 11, close_2_30pm_label % name_snowstorm),
        1996: (JUL, 5, close_1pm_label % name_day_after_independence_day),
        1997: (DEC, 26, close_1pm_label % name_friday_after_christmas),
        1999: (DEC, 31, close_1pm_label % "New Year's Eve"),
        2002: (JUL, 5, close_1pm_label % name_day_after_independence_day),
        2003: (DEC, 26, close_1pm_label % name_friday_after_christmas),
        2013: (JUL, 3, close_1pm_label % "Day before Independence Day"),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.countries.france import France
from holidays.mixins.child_entity import ChildEntity


class FrenchPolynesia(ChildEntity, France):
    """French Polynesia holidays.

    Alias of a French subdivision that is also officially assigned
    its own country code in ISO 3166-1.

    References:
        * <https://en.wikipedia.org/wiki/French_Polynesia>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_France>
    """

    country = "PF"
    parent_entity = France
    # Pōmare V abdicated and Tahiti became a French Colony on June 29th, 1880.
    start_year = 1881


class HolidaysPF(FrenchPolynesia):
    pass


class PF(FrenchPolynesia):
    pass


class PYF(FrenchPolynesia):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import APR, JUN, SEP, DEC
from holidays.constants import BANK, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_ONLY,
    MON_FRI_ONLY,
    TUE_TO_PREV_FRI,
    WED_TO_NEXT_FRI,
    FRI_ONLY,
    WORKDAY_TO_NEAREST_MON,
)


class Chile(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Chile holidays.

    References:
        * <https://web.archive.org/web/20250718012109/https://www.feriados.cl/>
        * [Excellent history of Chile holidays](https://web.archive.org/web/20250712031422/https://www.feriadoschilenos.cl/)
        * <https://es.wikipedia.org/wiki/Anexo:Días_feriados_en_Chile>
        * Law 2.977 (established official Chile holidays in its current form)
        * Law 20.983 (Day after New Year's Day, if it's a Sunday)
        * Law 19.668 (floating Monday holiday)
        * Law 19.668 (Corpus Christi)
        * Law 2.200, (Labour Day)
        * Law 18.018 (Labour Day renamed)
        * Law 16.840, Law 18.432 (Saint Peter and Saint Paul)
        * Law 20.148 (Day of Virgin of Carmen)
        * Law 18.026 (Day of National Liberation)
        * Law 19.588, Law 19.793 (Day of National Unity)
        * Law 20.983 (National Holiday Friday preceding Independence Day)
        * Law 20.215 (National Holiday Monday preceding Independence Day)
        * Law 20.215 (National Holiday Friday following Army Day)
        * Decree-law 636, Law 8.223
        * Law 3.810 (Columbus Day)
        * Law 20.299 (National Day of the Evangelical and Protestant Churches)
        * Law 20.663 (Región de Arica y Parinacota)
        * Law 20.678 (Región de Ñuble)
        * [Law 17.374 (holiday on census days)](https://web.archive.org/web/20250812022623/https://www.bcn.cl/leychile/navegar?idNorma=28960&idVersion=2002-02-01)
        * [Law 19.656 (Dec 31, 1999 holiday)](https://web.archive.org/web/20241228005823/https://www.bcn.cl/leychile/navegar?idNorma=149328&idVersion=1999-12-15)
        * [Law 12.051 (bank holidays Jun 30 and Dec 31)](https://web.archive.org/web/20241227190026/https://www.bcn.cl/leychile/navegar?idNorma=27013&idVersion=1956-07-12)
        * [Decree-law 1.171 (eliminate Jun 30)](https://web.archive.org/web/20241227191010/https://www.bcn.cl/leychile/navegar?idNorma=6507&idVersion=1975-09-05)
        * [Law 19.528 (eliminate Dec 31)](https://web.archive.org/web/20241227191452/https://www.bcn.cl/leychile/navegar?idNorma=76630&idVersion=1997-11-04)
        * [Law 19.559 (restore Dec 31)](https://web.archive.org/web/20241227195811/https://www.bcn.cl/leychile/navegar?idNorma=97758&idVersion=1998-04-16)
        * [Law 19.973 (Sep 17, 2004 holiday)](https://web.archive.org/web/20250812023003/https://www.bcn.cl/leychile/navegar?idLey=19973)
        * [Law 20.450 (Sep 17, 2010 and Sep 20, 2010 holidays)](https://web.archive.org/web/20250812023308/https://www.bcn.cl/leychile/navegar?idLey=20450)
        * [Law 21.521 (eliminate Dec 31 again, after the CMF publishes a specific regulation)](https://web.archive.org/web/20240214154900/https://www.bcn.cl/leychile/navegar?idNorma=1187323&idVersion=2023-02-03)
        * [Law 21.791 (restore bank holiday on Dec 31)](https://web.archive.org/web/20251219021727/https://www.bcn.cl/leychile/navegar?idNorma=1219578&idVersion=2025-12-17)
    """

    country = "CL"
    default_language = "es"
    start_year = 1915
    subdivisions = (
        "AI",  # Aisén del General Carlos Ibañez del Campo.
        "AN",  # Antofagasta.
        "AP",  # Arica y Parinacota.
        "AR",  # La Araucanía.
        "AT",  # Atacama.
        "BI",  # Biobío.
        "CO",  # Coquimbo.
        "LI",  # Libertador General Bernardo O'Higgins.
        "LL",  # Los Lagos.
        "LR",  # Los Ríos.
        "MA",  # Magallanes.
        "ML",  # Maule.
        "NB",  # Ñuble.
        "RM",  # Región Metropolitana de Santiago.
        "TA",  # Tarapacá.
        "VS",  # Valparaíso.
    )
    subdivisions_aliases = {
        "Aisén del General Carlos Ibañez del Campo": "AI",
        "Antofagasta": "AN",
        "Arica y Parinacota": "AP",
        "La Araucanía": "AR",
        "Atacama": "AT",
        "Biobío": "BI",
        "Coquimbo": "CO",
        "Libertador General Bernardo O'Higgins": "LI",
        "Los Lagos": "LL",
        "Los Ríos": "LR",
        "Magallanes": "MA",
        "Maule": "ML",
        "Ñuble": "NB",
        "Región Metropolitana de Santiago": "RM",
        "Tarapacá": "TA",
        "Valparaíso": "VS",
    }
    supported_categories = (BANK, PUBLIC)
    supported_languages = ("en_US", "es", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, ChileStaticHolidays)
        kwargs.setdefault("observed_rule", WORKDAY_TO_NEAREST_MON)
        kwargs.setdefault("observed_since", 2000)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Año Nuevo"))
        if self._year >= 2017:
            # National Holiday.
            self._add_observed(self._add_new_years_day_two(tr("Feriado nacional")), rule=MON_ONLY)

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Holy Saturday.
        self._add_holy_saturday(tr("Sábado Santo"))

        if self._year <= 1967:
            # Ascension Day.
            self._add_ascension_thursday(tr("Ascensión del Señor"))

        if self._year <= 1967 or 1987 <= self._year <= 2006:
            # Corpus Christi.
            name = tr("Corpus Christi")
            if self._year <= 1999:
                self._add_corpus_christi_day(name)
            else:
                self._add_holiday_57_days_past_easter(name)

        if self._year >= 1932:
            # Labor Day.
            self._add_labor_day(tr("Día Nacional del Trabajo"))

        # Naval Glories Day.
        self._add_holiday_may_21(tr("Día de las Glorias Navales"))

        if self._year >= 2021:
            # National Day of Indigenous Peoples.
            name = tr("Día Nacional de los Pueblos Indígenas")
            if self._year == 2021:
                self._add_holiday_jun_21(name)
            else:
                self._add_holiday(name, self._summer_solstice_date)

        if self._year <= 1967 or self._year >= 1986:
            # Saint Peter and Saint Paul's Day.
            self._move_holiday(self._add_saints_peter_and_paul_day(tr("San Pedro y San Pablo")))

        if self._year >= 2007:
            # Day of Virgin of Carmen.
            self._add_holiday_jul_16(tr("Virgen del Carmen"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Asunción de la Virgen"))

        if 1981 <= self._year <= 1998:
            # Day of National Liberation.
            self._add_holiday_sep_11(tr("Día de la Liberación Nacional"))
        elif 1999 <= self._year <= 2001:
            # Day of National Unity.
            self._add_holiday_1st_mon_of_sep(tr("Día de la Unidad Nacional"))

        # National Holiday.
        name = tr("Fiestas Patrias")

        if self._year >= 2007:
            self._add_observed(
                self._add_holiday_sep_17(name),
                rule=MON_FRI_ONLY if self._year >= 2017 else MON_ONLY,
            )
            if self._year >= 2008:
                self._add_observed(self._add_holiday_sep_20(name), rule=FRI_ONLY)
        elif 1932 <= self._year <= 1944:
            self._add_holiday_sep_20(name)

        # Independence Day.
        self._add_holiday_sep_18(tr("Día de la Independencia"))

        # Army Day.
        self._add_holiday_sep_19(tr("Día de las Glorias del Ejército"))

        if self._year >= 1922 and self._year != 1973:
            self._move_holiday(
                self._add_columbus_day(
                    # Meeting of Two Worlds' Day.
                    tr("Día del Encuentro de dos Mundos")
                    if self._year >= 2000
                    # Columbus Day.
                    else tr("Día de la Raza")
                )
            )

        if self._year >= 2008:
            # This holiday is moved to the preceding Friday if it falls on a Tuesday,
            # or to the following Friday if it falls on a Wednesday.
            self._move_holiday(
                self._add_holiday_oct_31(
                    # National Day of the Evangelical and Protestant Churches.
                    tr("Día Nacional de las Iglesias Evangélicas y Protestantes")
                ),
                rule=TUE_TO_PREV_FRI + WED_TO_NEXT_FRI,
            )

        # All Saints' Day.
        self._add_all_saints_day(tr("Día de Todos los Santos"))

        # Immaculate Conception.
        self._add_immaculate_conception_day(tr("La Inmaculada Concepción"))

        if 1944 <= self._year <= 1988:
            # Christmas Eve.
            self._add_christmas_eve(tr("Víspera de Navidad"))

        # Christmas Day.
        self._add_christmas_day(tr("Navidad"))

    def _populate_subdiv_ap_public_holidays(self):
        if self._year >= 2013:
            # Assault and Capture of Cape Arica.
            self._add_holiday_jun_7(tr("Asalto y Toma del Morro de Arica"))

    def _populate_subdiv_nb_public_holidays(self):
        if self._year >= 2014:
            self._add_holiday_aug_20(
                # Nativity of Bernardo O'Higgins (Chillán and Chillán Viejo communes)
                tr("Nacimiento del Prócer de la Independencia (Chillán y Chillán Viejo)")
            )

    def _populate_bank_holidays(self):
        # Bank Holiday.
        name = tr("Feriado bancario")
        if 1957 <= self._year <= 1975:
            self._add_holiday_jun_30(name)

        if 1956 <= self._year and self._year != 1997:
            self._add_holiday_dec_31(name)

    @property
    def _summer_solstice_date(self) -> tuple[int, int]:
        day = 20
        if (self._year % 4 > 1 and self._year <= 2046) or (
            self._year % 4 > 2 and self._year <= 2075
        ):
            day = 21
        return JUN, day


class CL(Chile):
    pass


class CHL(Chile):
    pass


class ChileStaticHolidays:
    # National Holiday.
    national_holiday = tr("Feriado nacional")

    # National Population and Housing Census.
    national_census = tr("Censo Nacional de Población y Vivienda")

    special_public_holidays = {
        1999: (DEC, 31, national_holiday),
        2002: (APR, 24, national_census),
        2004: (SEP, 17, national_holiday),
        2010: (
            (SEP, 17, national_holiday),
            (SEP, 20, national_holiday),
        ),
        2017: (APR, 19, national_census),
        2022: (SEP, 16, national_holiday),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, MAR, APR, MAY, JUL, AUG, SEP, OCT, DEC
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_WORKDAY


class Kazakhstan(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Kazakhstan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Kazakhstan>
        * <https://web.archive.org/web/20241225060133/https://egov.kz/cms/en/articles/holidays-calend>
        * <https://web.archive.org/web/20240828112119/https://adilet.zan.kz/kaz/docs/Z010000267_/history>
        * <https://web.archive.org/web/20240918145208/https://adilet.zan.kz/kaz/docs/Z990000493_#z63>

    Islamic holidays:
        * [2025](https://web.archive.org/web/20250429084936/https://qazinform.com/news/first-day-of-ramadan-to-fall-on-march-1-2025-ca393f)
    """

    country = "KZ"
    default_language = "kk"
    # %s (estimated).
    estimated_label = tr("%s (бағаланған)")
    # %s (observed).
    observed_label = tr("%s (қайта белгіленген демалыс)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (қайта белгіленген демалыс, бағаланған)")
    supported_languages = ("en_US", "kk", "uk")
    # Kazakhstan declared its sovereignty on 25 October 1990.
    start_year = 1991

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self, JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=KazakhstanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, KazakhstanStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 2002)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        name = tr("Жаңа жыл")
        dts_observed.add(self._add_new_years_day(name))
        dts_observed.add(self._add_new_years_day_two(name))

        if self._year >= 2006:
            # Orthodox Christmas.
            self._add_christmas_day(tr("Православиелік Рождество"))

        # International Women's Day.
        dts_observed.add(self._add_womens_day(tr("Халықаралық әйелдер күні")))

        if self._year >= 2002:
            # Nowruz Holiday.
            name = tr("Наурыз мейрамы")
            dts_observed.add(self._add_holiday_mar_22(name))
            if self._year >= 2010:
                dts_observed.add(self._add_holiday_mar_21(name))
                dts_observed.add(self._add_holiday_mar_23(name))

        # Kazakhstan's People Solidarity Holiday.
        dts_observed.add(self._add_labor_day(tr("Қазақстан халқының бірлігі мерекесі")))

        if self._year >= 2013:
            # Defender of the Fatherland Day.
            dts_observed.add(self._add_holiday_may_7(tr("Отан Қорғаушы күні")))

        # Victory Day.
        dts_observed.add(self._add_world_war_two_victory_day(tr("Жеңіс күні"), is_western=False))

        if self._year >= 2009:
            # Capital Day.
            dts_observed.add(self._add_holiday_jul_6(tr("Астана күні")))

        if self._year >= 1996:
            dts_observed.add(
                # Constitution Day.
                self._add_holiday_aug_30(tr("Қазақстан Республикасының Конституциясы күні"))
            )

        if 1994 <= self._year <= 2008 or self._year >= 2022:
            # Republic Day.
            dts_observed.add(self._add_holiday_oct_25(tr("Республика күні")))

        if 2012 <= self._year <= 2021:
            dts_observed.add(
                # First President Day.
                self._add_holiday_dec_1(tr("Қазақстан Республикасының Тұңғыш Президенті күні"))
            )

        # Independence Day.
        name = tr("Тəуелсіздік күні")
        dts_observed.add(self._add_holiday_dec_16(name))
        if 2002 <= self._year <= 2021:
            dts_observed.add(self._add_holiday_dec_17(name))

        if self.observed:
            self._populate_observed(dts_observed)

        if self._year >= 2006:
            # Eid al-Adha.
            self._add_eid_al_adha_day(tr("Құрбан айт"))


class KZ(Kazakhstan):
    pass


class KAZ(Kazakhstan):
    pass


class KazakhstanIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2006, 2025)
    EID_AL_ADHA_DATES = {
        2006: (JAN, 10),
        2015: (SEP, 24),
        2016: (SEP, 12),
    }


class KazakhstanStaticHolidays:
    """Kazakhstan special holidays.

    References:
        * [2000](https://web.archive.org/web/20250428203427/https://adilet.zan.kz/kaz/docs/P000000642_)
        * 2001:
            * <https://web.archive.org/web/20200221115328/http://adilet.zan.kz:80/kaz/docs/P010000282_>
            * <https://web.archive.org/web/20250428203415/https://adilet.zan.kz/kaz/docs/P010000515_>
            * <https://web.archive.org/web/20250428203411/https://adilet.zan.kz/kaz/docs/P010001604_>
        * [2002](https://web.archive.org/web/20250428203447/https://adilet.zan.kz/kaz/docs/P020000466_)
        * 2003:
            * <https://web.archive.org/web/20250428203424/https://adilet.zan.kz/kaz/docs/P030000338_>
            * <https://web.archive.org/web/20240913114351/https://www.adilet.zan.kz/kaz/docs/P030001166_>
        * 2005:
            * <https://web.archive.org/web/20180101044854/http://adilet.zan.kz:80/kaz/docs/P050000142_>
            * <https://web.archive.org/web/20250428203454/https://adilet.zan.kz/kaz/docs/P050000751_>
            * <https://web.archive.org/web/20250428204009/https://adilet.zan.kz/kaz/docs/P050000949_>
        * 2006:
            * <https://web.archive.org/web/20250428204118/https://adilet.zan.kz/kaz/docs/P050001309_>
            * <https://web.archive.org/web/20250428204001/https://adilet.zan.kz/kaz/docs/P060000277_>
        * 2007:
            * <https://web.archive.org/web/20250428204133/https://adilet.zan.kz/kaz/docs/P070000148_>
            * <https://web.archive.org/web/20250428204131/https://adilet.zan.kz/kaz/docs/P070000165_>
            * <https://web.archive.org/web/20250428204025/https://adilet.zan.kz/kaz/docs/P070000713_>
            * <https://web.archive.org/web/20250428204150/https://adilet.zan.kz/kaz/docs/P070000925_>
            * <https://web.archive.org/web/20250428204033/https://adilet.zan.kz/kaz/docs/P070001113_>
        * [2008](https://web.archive.org/web/20250428204634/https://adilet.zan.kz/kaz/docs/P080000364_)
        * [2009](https://web.archive.org/web/20250428204627/https://adilet.zan.kz/kaz/docs/P090001936_)
        * 2010:
            * <https://web.archive.org/web/20250428204638/https://adilet.zan.kz/kaz/docs/P090002216_>
            * <https://web.archive.org/web/20250428204659/https://adilet.zan.kz/kaz/docs/P100000637_>
        * 2011:
            * <https://web.archive.org/web/20170602150014/http://adilet.zan.kz:80/kaz/docs/P1100000167>
            * <https://web.archive.org/web/20250428204646/https://adilet.zan.kz/kaz/docs/P1100000948>
        * 2012:
            * <https://web.archive.org/web/20250429081405/https://adilet.zan.kz/kaz/docs/P1200000268>
            * <https://web.archive.org/web/20250428204720/https://adilet.zan.kz/kaz/docs/P1200000458>
            * <https://web.archive.org/web/20250428204647/https://adilet.zan.kz/kaz/docs/P1200001538>
        * 2013:
            * <https://web.archive.org/web/20130424070807/http://adilet.zan.kz/kaz/docs/P1300000345>
            * <https://web.archive.org/web/20250428204654/https://adilet.zan.kz/kaz/docs/P1300001068>
            * <https://web.archive.org/web/20131213045017/http://adilet.zan.kz/kaz/docs/P1300001322>
        * [2014](https://web.archive.org/web/20200715173002/http://adilet.zan.kz/kaz/docs/P1400000365)
        * [2016](https://web.archive.org/web/20200225220136/http://adilet.zan.kz/kaz/docs/P1600000067)
        * [2017](https://web.archive.org/web/20200203183402/http://adilet.zan.kz/kaz/docs/P1700000005)
        * [2018](https://web.archive.org/web/20200226012057/http://adilet.zan.kz/kaz/docs/P1700000864)
        * [2019](https://web.archive.org/web/20200630083024/http://adilet.zan.kz/kaz/docs/P1800000888)
        * [2020](https://web.archive.org/web/20221023205506/https://adilet.zan.kz/kaz/docs/P1900000820)
        * [2021](https://web.archive.org/web/20250428204757/https://adilet.zan.kz/kaz/docs/P2000000930)
        * [2022](https://web.archive.org/web/20250428204753/https://adilet.zan.kz/kaz/docs/P2200000796)
        * [2023](https://web.archive.org/web/20230928205344/https://adilet.zan.kz/kaz/docs/P2300000326)
        * [2024](https://web.archive.org/web/20250108222408/https://adilet.zan.kz/kaz/docs/G24G0000109)
        * [2025](https://web.archive.org/web/20250428203311/https://adilet.zan.kz/kaz/docs/G24G0000436)
    """

    # Substituted date format.
    substituted_date_format = tr("%d.%m.%Y")
    # Day off (substituted from %s).
    substituted_label = tr("Демалыс күні (%s бастап ауыстырылды)")

    special_public_holidays = {
        2000: (MAY, 8, MAY, 6),
        2001: (
            (MAR, 9, MAR, 11),
            (MAR, 23, MAR, 25),
            (APR, 30, APR, 28),
            (DEC, 31, DEC, 29),
        ),
        2002: (MAY, 10, MAY, 12),
        2003: (
            (MAY, 2, MAY, 4),
            (DEC, 15, DEC, 13),
        ),
        2005: (
            (MAR, 7, MAR, 5),
            (MAR, 21, MAR, 19),
            (AUG, 29, AUG, 27),
            (OCT, 24, OCT, 22),
        ),
        2006: (
            (JAN, 11, JAN, 14),
            (MAY, 8, MAY, 6),
        ),
        2007: (
            (MAR, 9, MAR, 11),
            (MAR, 23, MAR, 25),
            (AUG, 31, SEP, 2),
            (OCT, 26, OCT, 28),
            (DEC, 31, DEC, 29),
        ),
        2008: (MAY, 2, MAY, 4),
        2009: (DEC, 18, DEC, 20),
        2010: (
            (JAN, 8, JAN, 10),
            (JUL, 5, JUL, 3),
        ),
        2011: (
            (MAR, 7, MAR, 5),
            (AUG, 29, AUG, 27),
        ),
        2012: (
            (MAR, 9, MAR, 11),
            (APR, 30, APR, 28),
            (DEC, 31, DEC, 29),
        ),
        2013: (
            (MAY, 10, MAY, 4),
            (OCT, 14, OCT, 12),
        ),
        2014: (
            (JAN, 3, DEC, 28, 2013),
            (MAY, 2, MAY, 4),
            (MAY, 8, MAY, 11),
        ),
        2016: (MAR, 7, MAR, 5),
        2017: (
            (MAR, 20, MAR, 18),
            (JUL, 7, JUL, 1),
        ),
        2018: (
            (MAR, 9, MAR, 3),
            (APR, 30, APR, 28),
            (MAY, 8, MAY, 5),
            (AUG, 31, AUG, 25),
            (DEC, 31, DEC, 29),
        ),
        2019: (MAY, 10, MAY, 4),
        2020: (
            (JAN, 3, JAN, 5),
            (MAY, 8, MAY, 11),
            (DEC, 18, DEC, 20),
        ),
        2021: (JUL, 5, JUL, 3),
        2022: (
            (MAR, 7, MAR, 5),
            (AUG, 29, AUG, 27),
            (OCT, 24, OCT, 22),
        ),
        2023: (JUL, 7, JUL, 1),
        2024: (MAY, 8, MAY, 4),
        2025: (JAN, 3, JAN, 5),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, THU, FRI, SAT
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Jordan(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Jordan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Jordan>
        * <https://web.archive.org/web/20250116061815/https://www.mfa.gov.jo/content/public-holidays>
    """

    country = "JO"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (المقدرة)")
    supported_languages = ("ar", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # The resting days are Friday and Saturday since Jan 6, 2000.
        # https://web.archive.org/web/20241226195649/http://archive.wfn.org/2000/01/msg00078.html
        return {FRI, SAT} if dt >= date(2000, JAN, 6) else {THU, FRI}

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("رأس السنة الميلادية"))

        # Labor Day.
        self._add_labor_day(tr("عيد العمال"))

        # Independence Day.
        self._add_holiday_may_25(tr("عيد الإستقلال"))

        # Christmas Day.
        self._add_christmas_day(tr("عيد الميلاد المجيد"))

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("عيد المولد النبوي"))

        # Isra' and Mi'raj.
        self._add_isra_and_miraj_day(tr("ليلة المعراج"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("عيد الفطر"))
        # Eid al-Fitr Holiday.
        name = tr("عطلة عيد الفطر")
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Arafat Day.
        self._add_arafah_day(tr("يوم عرفة"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("عيد الأضحى"))
        # Eid al-Adha Holiday.
        name = tr("عطلة عيد الأضحى")
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)


class JO(Jordan):
    pass


class JOR(Jordan):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars.gregorian import (
    JAN,
    MAY,
    JUN,
    JUL,
    AUG,
    SEP,
    OCT,
    DEC,
    SUN,
    _timedelta,
    CHRISTMAS,
    WINTER_SOLSTICE,
)
from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import (
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    StaticHolidays,
)
from holidays.mixins.preferred_discretionary_holidays import PreferredDiscretionaryHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_WORKDAY,
    SUN_TO_NEXT_WORKDAY,
    WORKDAY_TO_NEXT_WORKDAY,
)


class HongKong(
    ObservedHolidayBase,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    PreferredDiscretionaryHolidays,
    StaticHolidays,
):
    """Hong Kong holidays.

    References:
        * [English Wikipedia](https://en.wikipedia.org/wiki/Public_holidays_in_Hong_Kong)
        * [Chinese Wikipedia](https://zh.wikipedia.org/wiki/香港節日與公眾假期)

    Statutory Holidays:
        * [Section 39 of Cap. 57 Employment Ordinance](http://archive.today/2025.04.29-125537/https://www.elegislation.gov.hk/hk/cap57!en-zh-Hant-HK?INDEX_CS=N&xpid=ID_1438403463460_002)
        * [Holidays for 2010-2025](https://web.archive.org/web/20250402133857/https://www.labour.gov.hk/eng/news/holidays_list.htm)

    General Holidays:
        * [Cap. 149 General Holidays Ordinance](http://archive.today/2025.04.29-125900/https://www.elegislation.gov.hk/hk/cap149!en-zh-Hant-HK?INDEX_CS=N)
        * [Holidays for 2007-2025](https://web.archive.org/web/20250116080351/https://www.gov.hk/en/about/abouthk/holiday/index.htm)
    """

    country = "HK"
    default_language = "zh_HK"
    default_preferred_discretionary_holidays = (CHRISTMAS,)
    # %s (estimated).
    estimated_label = tr("%s（推定）")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s（補假，推定）")
    # %s (observed).
    observed_label = tr("%s（補假）")
    supported_categories = (OPTIONAL, PUBLIC)
    supported_languages = ("en_HK", "en_US", "th", "zh_CN", "zh_HK")
    weekend = {SUN}
    # Current set of holidays actually valid since 1946
    start_year = 1946

    def __init__(self, *args, **kwargs):
        ChineseCalendarHolidays.__init__(self)
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        PreferredDiscretionaryHolidays.__init__(
            self, kwargs.pop("preferred_discretionary_holidays", None)
        )
        StaticHolidays.__init__(self, HongKongStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_WORKDAY)
        super().__init__(*args, **kwargs)

    def _add_mid_autumn(self) -> date:
        # Mid-Autumn Festival.

        mid_autumn_date = self._mid_autumn_festival
        if self._year >= 1968:
            mid_autumn_date = _timedelta(mid_autumn_date, +1)
            # The Day following Mid-Autumn Festival.
            name = tr("中秋節翌日")
            # The Second Day following Mid-Autumn Festival.
            second_name = tr("中秋節後第二日")
        else:
            # Mid-Autumn Festival.
            name = tr("中秋節")
            # The Day following Mid-Autumn Festival.
            second_name = tr("中秋節翌日")

        if self._is_sunday(mid_autumn_date):
            if 1983 <= self._year <= 2010:
                # Mid-Autumn Festival.
                self._add_holiday(tr("中秋節"), _timedelta(mid_autumn_date, -1))
            else:
                self._add_holiday(second_name, _timedelta(mid_autumn_date, +1))
        else:
            self._add_holiday(name, mid_autumn_date)
        return mid_autumn_date

    def _add_lunar_new_year(self, day_three_start_year: int):
        # Chinese New Year.
        name = tr("農曆年初一")
        # Chinese New Year's Eve.
        preceding_day_lunar = tr("農曆年初一的前一日")
        # The second day of Chinese New Year.
        second_day_lunar = tr("農曆年初二")
        # The third day of Chinese New Year.
        third_day_lunar = tr("農曆年初三")
        # The fourth day of Chinese New Year.
        fourth_day_lunar = tr("農曆年初四")
        dt_lunar_new_year = self._chinese_new_year
        if self._year >= 1983:
            if (
                self._is_friday(dt_lunar_new_year)
                or self._is_saturday(dt_lunar_new_year)
                or self._is_sunday(dt_lunar_new_year)
            ):
                if self._year >= 2012:
                    self._add_chinese_new_years_day_four(fourth_day_lunar)
                else:
                    self._add_chinese_new_years_eve(preceding_day_lunar)

            if not self._is_sunday(dt_lunar_new_year):
                self._add_chinese_new_years_day(name)

            if not self._is_saturday(dt_lunar_new_year):
                self._add_chinese_new_years_day_two(second_day_lunar)

            if not self._is_friday(dt_lunar_new_year):
                self._add_chinese_new_years_day_three(third_day_lunar)
        else:
            self._add_chinese_new_years_day(name)
            self._add_chinese_new_years_day_two(second_day_lunar)
            if self._year >= day_three_start_year:
                self._add_chinese_new_years_day_three(third_day_lunar)

    def _populate_public_holidays(self):
        # Statutory Holidays.

        # Industrial Employment Ordinance implemented in April 1962.
        if self._year <= 1962:
            return None

        if self._year >= 1977:
            # New Year's Day.
            self._add_observed(self._add_new_years_day(tr("一月一日")))

        self._add_lunar_new_year(day_three_start_year=1977)

        if self._year >= 2028:
            # Good Friday.
            self._add_good_friday(tr("耶穌受難節"))

        if self._year >= 2030:
            # The day following Good Friday.
            self._add_holy_saturday(tr("耶穌受難節翌日"))

        if self._year >= 2026:
            # Easter Monday.
            self._add_easter_monday(tr("復活節星期一"))

        # Tomb-Sweeping Day.
        self._add_observed(self._add_qingming_festival(tr("清明節")))

        if self._year >= 2022:
            # The Buddha's Birthday.
            self._add_observed(self._add_chinese_birthday_of_buddha(tr("佛誕")))

        if self._year >= 1999:
            # Labor Day.
            self._add_observed(self._add_labor_day(tr("勞動節")))

        # Dragon Boat Festival.
        self._add_observed(self._add_dragon_boat_festival(tr("端午節")))

        if self._year >= 1997:
            # Hong Kong S.A.R. Establishment Day.
            self._add_observed(self._add_holiday_jul_1(tr("香港特別行政區成立紀念日")))

        mid_autumn_date = self._add_mid_autumn()

        if self._year >= 1977:
            # Double Ninth Festival.
            dt_double_ninth = self._add_double_ninth_festival(tr("重陽節"))
            self._add_observed(dt_double_ninth)

        if self._year >= 1997:
            # National Day.
            name = tr("國慶日")
            oct_1 = self._add_holiday_oct_1(name)
            self._add_observed(
                oct_1,
                name=name,
                rule=WORKDAY_TO_NEXT_WORKDAY + SAT_SUN_TO_NEXT_WORKDAY
                if oct_1 == mid_autumn_date or oct_1 == dt_double_ninth
                else SUN_TO_NEXT_WORKDAY,
            )

        if WINTER_SOLSTICE in self.preferred_discretionary_holidays:
            # Winter Solstice.
            self._add_observed(self._add_dongzhi_festival(tr("冬節")))

        if self._year >= 2024:
            # The first weekday after Christmas Day.
            self._add_observed(self._add_christmas_day_two(tr("聖誕節後第一個周日")))

        if CHRISTMAS in self.preferred_discretionary_holidays:
            # Christmas Day.
            self._add_observed(self._add_christmas_day(tr("聖誕節")))

    def _populate_optional_holidays(self):
        # General Holidays.

        if self._is_sunday(JAN, 1):
            # The day following New Year's Day.
            self._add_new_years_day_two(tr("一月一日翌日"))
        else:
            # New Year's Day.
            self._add_new_years_day(tr("一月一日"))

        self._add_lunar_new_year(day_three_start_year=1968)

        if self._year >= 1968:
            dt_qingming = self._qingming_festival
            if self._is_sunday(dt_qingming) or dt_qingming == _timedelta(self._easter_sunday, +1):
                # The day following Tomb-Sweeping Day.
                self._add_holiday(tr("清明節翌日"), _timedelta(dt_qingming, +1))
            else:
                # Tomb-Sweeping Day.
                self._add_qingming_festival(tr("清明節"))

        # Good Friday.
        self._add_good_friday(tr("耶穌受難節"))

        # The day following Good Friday.
        self._add_holy_saturday(tr("耶穌受難節翌日"))

        if self._year >= 1968 and dt_qingming == self._easter_sunday:
            # The day following Easter Monday.
            self._add_easter_tuesday(tr("復活節星期一翌日"))
        else:
            # Easter Monday.
            self._add_easter_monday(tr("復活節星期一"))

        if self._year >= 1999:
            dt_birthday_of_buddha = self._chinese_birthday_of_buddha
            if self._is_sunday(dt_birthday_of_buddha):
                # The day following the Buddha's Birthday.
                self._add_holiday(tr("佛誕翌日"), _timedelta(dt_birthday_of_buddha, +1))
            else:
                # The Buddha's Birthday.
                self._add_chinese_birthday_of_buddha(tr("佛誕"))

        if self._year >= 1999:
            if self._is_sunday(MAY, 1):
                # The day following Labor Day.
                self._add_labor_day_two(tr("勞動節翌日"))
            else:
                # Labor Day.
                self._add_labor_day(tr("勞動節"))

        if self._year >= 1968:
            dt_dragon_boat = self._dragon_boat_festival
            if self._is_sunday(dt_dragon_boat):
                # The day following Dragon Boat Festival.
                self._add_holiday(tr("端午節翌日"), _timedelta(dt_dragon_boat, +1))
            else:
                # Dragon Boat Festival.
                self._add_dragon_boat_festival(tr("端午節"))

        if self._year >= 1997:
            if self._is_sunday(JUL, 1):
                # The day following Hong Kong S.A.R. Establishment Day.
                self._add_holiday_jul_2(tr("香港特別行政區成立紀念日翌日"))
            else:
                # Hong Kong S.A.R. Establishment Day.
                self._add_holiday_jul_1(tr("香港特別行政區成立紀念日"))

        mid_autumn_date = self._add_mid_autumn()

        if self._year >= 1968:
            dt_double_ninth = self._double_ninth_festival
            if self._is_sunday(dt_double_ninth):
                # The day following Double Ninth Festival.
                self._add_holiday(tr("重陽節翌日"), _timedelta(dt_double_ninth, +1))
            else:
                # Double Ninth Festival.
                self._add_double_ninth_festival(tr("重陽節"))

        if self._year >= 1997:
            dt = date(self._year, OCT, 1)
            # The day following National Day.
            name = tr("國慶日翌日")
            if self._is_sunday(dt) or dt == mid_autumn_date or dt == dt_double_ninth:
                self._add_holiday(name, self._get_next_workday(dt))
            else:
                # National Day.
                self._add_holiday_oct_1(tr("國慶日"))

            if self._year <= 1998:
                self._add_holiday_oct_2(name)

        # Christmas Day.
        name = tr("聖誕節")
        # The first weekday after Christmas Day.
        first_after_christmas = tr("聖誕節後第一個周日")
        # The second weekday after Christmas Day.
        second_after_christmas = tr("聖誕節後第二個周日")
        dt_christmas = self._christmas_day
        if self._is_sunday(dt_christmas):
            self._add_christmas_day_two(first_after_christmas)
            self._add_christmas_day_three(second_after_christmas)
        elif self._is_saturday(dt_christmas):
            self._add_christmas_day(name)
            self._add_christmas_day_three(first_after_christmas)
        else:
            self._add_christmas_day(name)
            self._add_christmas_day_two(first_after_christmas)

        # Previous holidays.

        if 1952 <= self._year <= 1996:
            # Queen's Birthday.
            name = tr("英女皇壽辰")
            if self._year >= 1983:
                self._add_holiday_2nd_sat_of_jun(name)
                self._add_holiday_2_days_past_2nd_sat_of_jun(name)
            else:
                if self._year != 1952:
                    self._add_holiday_apr_21(name)
                else:
                    self._add_holiday_jun_5(name)

        if self._year <= 1967:
            # Monday after Pentecost.
            self._add_whit_monday(tr("靈降臨節後星期一"))

            # National Day of the Republic of China.
            self._add_holiday_2nd_mon_of_oct(tr("中華民國國慶日"))

            # Monday after Remembrance Day.
            self._add_holiday_1_day_past_2nd_sun_of_nov(tr("和平紀念日後星期一"))

        if self._year <= 1996:
            # Anniversary of the liberation of Hong Kong.
            name = tr("重光紀念日")
            if self._year >= 1983:
                self._add_holiday_last_mon_of_aug(name)
                self._add_holiday_2_days_prior_last_mon_of_aug(name)
            elif self._year >= 1968:
                self._add_holiday_1st_mon_of_aug(name)
                self._add_holiday_last_mon_of_aug(name)
            else:
                self._add_holiday_aug_30(name)


class HK(HongKong):
    pass


class HKG(HongKong):
    pass


class HongKongStaticHolidays:
    # Wedding of Prince Charles and Diana.
    wedding_of_charles_and_diana = tr("英國王儲查理斯王子與戴安娜婚禮")

    # Second day of Queen Elizabeth II and her husband's visit to Hong Kong.
    queen_visit_hk = tr("英女王伊利沙伯二世伉儷訪港的第二天")

    # Queen's Birthday.
    queen_birthday = tr("英女皇壽辰")

    # The day following Hong Kong S.A.R. Establishment Day.
    day_following_hksar_establishment_day = tr("香港特別行政區成立紀念日翌日")

    # 70th Anniversary of the Victory of the Chinese People's War of Resistance against
    # Japanese Aggression and the World Anti-Fascist War.
    victory_70th_anniversary = tr("中國人民抗日戰爭勝利70周年紀念日")

    # Sino-Japanese War Victory Day.
    war_victory_day = tr("抗日戰爭勝利紀念日")

    # The day following National Day.
    day_following_national_day = tr("國慶日翌日")

    # Additional public holiday.
    additional_public_holiday = tr("額外公眾假期")

    special_public_holidays = {
        1981: (JUL, 29, wedding_of_charles_and_diana),
        1986: (OCT, 22, queen_visit_hk),
        1997: (JUL, 2, day_following_hksar_establishment_day),
        2015: (SEP, 3, victory_70th_anniversary),
    }

    special_optional_holidays = {
        1981: (JUL, 29, wedding_of_charles_and_diana),
        1986: (OCT, 22, queen_visit_hk),
        1997: (
            (JUN, 28, queen_birthday),
            (JUN, 30, queen_birthday),
            (JUL, 2, day_following_hksar_establishment_day),
            (AUG, 18, war_victory_day),
            (OCT, 2, day_following_national_day),
        ),
        1998: (
            (AUG, 17, war_victory_day),
            (OCT, 2, day_following_national_day),
        ),
        1999: (DEC, 31, additional_public_holiday),
        2015: (SEP, 3, victory_70th_anniversary),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.gregorian import MAR, JUN, JUL, SEP, _timedelta
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    ALL_TO_NEAREST_MON,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class NewZealand(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """New Zealand holidays."""

    country = "NZ"
    # %s (observed).
    observed_label = "%s (observed)"
    start_year = 1894
    subdivisions = (
        # ISO 3166-2: Regions and Special Island Authorities.
        # https://en.wikipedia.org/wiki/ISO_3166-2:NZ
        "AUK",  # Auckland (Tāmaki-Makaurau).
        "BOP",  # Bay of Plenty (Toi Moana).
        "CAN",  # Canterbury (Waitaha).
        "CIT",  # Chatham Islands Territory (Wharekauri).
        "GIS",  # Gisborne (Te Tairāwhiti).
        "HKB",  # Hawke's Bay (Te Matau-a-Māui).
        "MBH",  # Marlborough (Manawatū-Whanganui).
        "MWT",  # Manawatū-Whanganui (Manawatū Whanganui).
        "NSN",  # Nelson (Whakatū).
        "NTL",  # Northland (Te Taitokerau).
        "OTA",  # Otago (Ō Tākou).
        "STL",  # Southland (Te Taiao Tonga).
        "TAS",  # Tasman (Te tai o Aorere).
        "TKI",  # Taranaki (Taranaki).
        "WGN",  # Greater Wellington (Te Pane Matua Taiao).
        "WKO",  # Waikato (Waikato).
        "WTC",  # West Coast (Te Tai o Poutini).
        # Subregions:
        # https://web.archive.org/web/20250415230521/https://www.govt.nz/browse/work/public-holidays-and-work/public-holidays-and-anniversary-dates/
        "South Canterbury",
    )
    subdivisions_aliases = {
        # Fullnames in English and Maori, as well as HASC.
        "Auckland": "AUK",
        "Tāmaki-Makaurau": "AUK",
        "AU": "AUK",
        "Bay of Plenty": "BOP",
        "Toi Moana": "BOP",
        "BP": "BOP",
        "Canterbury": "CAN",
        "Waitaha": "CAN",
        "CA": "CAN",
        "Chatham Islands Territory": "CIT",
        "Chatham Islands": "CIT",  # 1901-1994, as County
        "Wharekauri": "CIT",
        "CI": "CIT",
        "Gisborne": "GIS",
        "Te Tairāwhiti": "GIS",
        "GI": "GIS",
        "Hawke's Bay": "HKB",
        "Te Matau-a-Māui": "HKB",
        "HB": "HKB",
        "Marlborough": "MBH",
        "MA": "MBH",
        "Manawatū Whanganui": "MWT",
        "Manawatū-Whanganui": "MWT",
        "MW": "MWT",
        "Nelson": "NSN",
        "Whakatū": "NSN",
        "NE": "NSN",
        "Northland": "NTL",
        "Te Taitokerau": "NTL",
        "NO": "NTL",
        "Otago": "OTA",
        "Ō Tākou": "OTA",
        "OT": "OTA",
        "Southland": "STL",
        "Te Taiao Tonga": "STL",
        "SO": "STL",
        "Tasman": "TAS",
        "Te tai o Aorere": "TAS",
        "TS": "TAS",
        "Taranaki": "TKI",
        "TK": "TKI",
        "Greater Wellington": "WGN",
        "Te Pane Matua Taiao": "WGN",
        "Wellington": "WGN",  # Prev. ISO code from 2010-2015.
        "Te Whanganui-a-Tara": "WGN",  # Prev. ISO code from 2010-2015.
        "WG": "WGN",
        "Waikato": "WKO",
        "WK": "WKO",
        "West Coast": "WTC",
        "Te Tai o Poutini": "WTC",
        "WC": "WTC",
    }
    _deprecated_subdivisions = (
        # Pre-1893 Naming in Previous Implementations.
        "New Plymouth",  # 1853-1859, Now Taranaki.
        "Westland",  # 1873-1876, Now West Coast.
        # Unofficial Code.
        "STC",  # For 'South Canterbury' Subregional Holidays.
        "WTL",  # Westland, Correct code is WTC (for West Coast).
    )

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, NewZelandStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Bank Holidays Act 1873
        # The Employment of Females Act 1873
        # Factories Act 1894
        # Industrial Conciliation and Arbitration Act 1894
        # Labour Day Act 1899
        # Anzac Day Act 1920, 1949, 1956
        # New Zealand Day Act 1973
        # Waitangi Day Act 1960, 1976
        # Sovereign's Birthday Observance Act 1937, 1952
        # Holidays Act 1981, 2003

        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)
        self._add_observed(
            # Day after New Year's Day.
            self._add_new_years_day_two("Day after New Year's Day"),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        if self._year >= 1974:
            feb_6 = self._add_holiday_feb_6(
                # Waitangi Day.
                "Waitangi Day"
                if self._year >= 1977
                # New Zealand Day.
                else "New Zealand Day"
            )
            if self._year >= 2014:
                self._add_observed(feb_6)

        if self._year >= 1921:
            # Anzac Day.
            apr_25 = self._add_anzac_day("Anzac Day")
            if self._year >= 2014:
                self._add_observed(apr_25)

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Sovereign's Birthday
        if self._year >= 1902:
            name = (
                # Queen's Birthday.
                "Queen's Birthday"
                if 1952 <= self._year <= 2022
                # King's Birthday.
                else "King's Birthday"
            )
            if self._year == 1952:
                self._add_holiday_jun_2(name)  # Elizabeth II
            elif self._year >= 1938:
                self._add_holiday_1st_mon_of_jun(name)  # EII & GVI
            elif self._year == 1937:
                self._add_holiday_jun_9(name)  # George VI
            elif self._year == 1936:
                self._add_holiday_jun_23(name)  # Edward VIII
            elif self._year >= 1912:
                self._add_holiday_jun_3(name)  # George V
            else:
                # https://web.archive.org/web/20250427125629/https://paperspast.natlib.govt.nz/cgi-bin/paperspast?a=d&d=NZH19091110.2.67
                self._add_holiday_nov_9(name)  # Edward VII

        # Matariki
        dates_obs = {
            2022: (JUN, 24),
            2023: (JUL, 14),
            2024: (JUN, 28),
            2025: (JUN, 20),
            2026: (JUL, 10),
            2027: (JUN, 25),
            2028: (JUL, 14),
            2029: (JUL, 6),
            2030: (JUN, 21),
            2031: (JUL, 11),
            2032: (JUL, 2),
            2033: (JUN, 24),
            2034: (JUL, 7),
            2035: (JUN, 29),
            2036: (JUL, 18),
            2037: (JUL, 10),
            2038: (JUN, 25),
            2039: (JUL, 15),
            2040: (JUL, 6),
            2041: (JUL, 19),
            2042: (JUL, 11),
            2043: (JUL, 3),
            2044: (JUN, 24),
            2045: (JUL, 7),
            2046: (JUN, 29),
            2047: (JUL, 19),
            2048: (JUL, 3),
            2049: (JUN, 25),
            2050: (JUL, 15),
            2051: (JUN, 30),
            2052: (JUN, 21),
        }
        if dt := dates_obs.get(self._year):
            # Matariki.
            self._add_holiday("Matariki", dt)

        if self._year >= 1900:
            # Labour Day.
            name = "Labour Day"
            if self._year >= 1910:
                self._add_holiday_4th_mon_of_oct(name)
            else:
                self._add_holiday_2nd_wed_of_oct(name)

        # Christmas Day.
        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two("Boxing Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)

        if self.subdiv == "New Plymouth":
            self._populate_subdiv_tki_public_holidays()
        elif self.subdiv == "STC":
            self._populate_subdiv_south_canterbury_public_holidays()
        elif self.subdiv in {"WTL", "Westland"}:
            self._populate_subdiv_wtc_public_holidays()

    def _populate_subdiv_auk_public_holidays(self):
        self._move_holiday_forced(
            # Auckland Anniversary Day.
            self._add_holiday_jan_29("Auckland Anniversary Day"),
            rule=ALL_TO_NEAREST_MON,
        )

    def _populate_subdiv_can_public_holidays(self):
        # Canterbury Anniversary Day.
        self._add_holiday_10_days_past_1st_tue_of_nov("Canterbury Anniversary Day")

    def _populate_subdiv_cit_public_holidays(self):
        self._move_holiday_forced(
            # Chatham Islands Anniversary Day.
            self._add_holiday_nov_30("Chatham Islands Anniversary Day"),
            rule=ALL_TO_NEAREST_MON,
        )

    def _populate_subdiv_hkb_public_holidays(self):
        # Hawke's Bay Anniversary Day.
        self._add_holiday_3_days_prior_4th_mon_of_oct("Hawke's Bay Anniversary Day")

    def _populate_subdiv_mbh_public_holidays(self):
        # Marlborough Anniversary Day.
        self._add_holiday_7_days_past_4th_mon_of_oct("Marlborough Anniversary Day")

    def _populate_subdiv_nsn_public_holidays(self):
        self._move_holiday_forced(
            # Nelson Anniversary Day.
            self._add_holiday_feb_1("Nelson Anniversary Day"),
            rule=ALL_TO_NEAREST_MON,
        )

    def _populate_subdiv_ntl_public_holidays(self):
        if 1964 <= self._year <= 1973:
            self._move_holiday_forced(
                # Waitangi Day.
                self._add_holiday_feb_6("Waitangi Day"),
                rule=ALL_TO_NEAREST_MON,
            )
        else:
            self._move_holiday_forced(
                # Auckland Anniversary Day.
                self._add_holiday_jan_29("Auckland Anniversary Day"),
                rule=ALL_TO_NEAREST_MON,
            )

    def _populate_subdiv_ota_public_holidays(self):
        # there is no easily determined single day of local observance?!?!
        # If Easter Monday, observed on Easter Tuesday instead.
        dt = self._get_observed_date(date(self._year, MAR, 23), rule=ALL_TO_NEAREST_MON)
        if dt == _timedelta(self._easter_sunday, +1):
            dt = _timedelta(dt, +1)
        # Otago Anniversary Day.
        self._add_holiday("Otago Anniversary Day", dt)

    def _populate_subdiv_south_canterbury_public_holidays(self):
        # South Canterbury Anniversary Day.
        self._add_holiday_4th_mon_of_sep("South Canterbury Anniversary Day")

    def _populate_subdiv_stl_public_holidays(self):
        # Southland Anniversary Day.
        name = "Southland Anniversary Day"
        if self._year >= 2012:
            self._add_easter_tuesday(name)
        else:
            self._move_holiday_forced(self._add_holiday_jan_17(name), rule=ALL_TO_NEAREST_MON)

    def _populate_subdiv_tki_public_holidays(self):
        # Taranaki Anniversary Day.
        self._add_holiday_2nd_mon_of_mar("Taranaki Anniversary Day")

    def _populate_subdiv_wgn_public_holidays(self):
        self._move_holiday_forced(
            # Wellington Anniversary Day.
            self._add_holiday_jan_22("Wellington Anniversary Day"),
            rule=ALL_TO_NEAREST_MON,
        )

    def _populate_subdiv_wtc_public_holidays(self):
        # West Coast Anniversary Day.
        name = "West Coast Anniversary Day"
        if self._year == 2005:
            self._add_holiday_dec_5(name)
        else:
            self._move_holiday_forced(self._add_holiday_dec_1(name), rule=ALL_TO_NEAREST_MON)


class NZ(NewZealand):
    pass


class NZL(NewZealand):
    pass


class NewZelandStaticHolidays:
    special_public_holidays = {
        # Queen Elizabeth II Memorial Day.
        2022: (SEP, 26, "Queen Elizabeth II Memorial Day"),
    }
